/* devicedetails.cc
 * Copyright (C) 2001 The University of Newcastle upon Tyne
 *
 * Written by Paul Coates (Paul.Coates@ncl.ac.uk)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 */

#include <gtk/gtk.h>
#include <unistd.h>
#include <string.h>
#include <stdio.h>
#include <pwd.h>
#include "devicedetails.h"
#include "misc.h"
#include "main.h"
#include "features.h"
#include "gui.h"
#include "ipaddress.h"
#include "device.h"
#include "data.h"
#include "session.h"
#include "scheduler.h"
#include "licon.xpm"
#include "intup.xpm"
#include "intdown.xpm"
#include "lrouter.xpm"
#include "lswitch.xpm"
#include "lserver.xpm"
#include "lcomputer.xpm"

static void select_row_cb(GtkWidget *clist, gint row, gint column, GdkEventButton *event, gpointer data)
{ ((DeviceDetails*)data)->select_row(row); }
static void unselect_row_cb(GtkWidget *clist, gint row, gint column, GdkEventButton *event, gpointer data)
{ ((DeviceDetails*)data)->unselect_row(row); }
static void ok_cb(GtkWidget *w, gpointer data)
{ ((DeviceDetails*)data)->save(); ((DeviceDetails*)data)->hide(); }
static void cancel_cb(GtkWidget *w, gpointer data)
{ ((DeviceDetails*)data)->hide(); }
static gint delete_event(GtkWidget *widget, GdkEvent *event, gpointer data)
{ ((DeviceDetails*)data)->hide(); return TRUE; }
static void combo_cb(GtkWidget *w, gpointer data)
{
	int i;
	char *nm;
	DeviceDetails *d = (DeviceDetails*) data;
	nm = gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(d->combo2)->entry));
//	printf("COMBO %s\n", nm);
	if (strcmp(nm, "Switch")==0) i=1; else
	if (strcmp(nm, "Bridge")==0) i=2; else
	if (strcmp(nm, "Hub")==0) i=3; else i=0;
	gtk_notebook_set_page(GTK_NOTEBOOK (d->notebook1), i);
}

DeviceDetails::DeviceDetails(Main *m)
{
	main = m;
	device = NULL;

  GtkWidget *vbox66;
  GtkWidget *table4;
  GtkWidget *hbox85;
  GtkWidget *scrolledwindow33;
  GtkWidget *label175;
  GtkWidget *label176;
  GtkWidget *label177;
  GtkWidget *label178;
  GtkWidget *label179;
  GtkWidget *scrolledwindow32;
  GtkWidget *label174;
  GtkWidget *hbox87;
  GtkWidget *hseparator4;
  GtkWidget *hbox84;
  GtkWidget *button92;
  GtkWidget *button93;
  GList *combo2_items = NULL;

  window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_window_set_title (GTK_WINDOW (window), "Device Details");
	gtk_widget_realize(window);
	window_icon(window, licon_xpm);
	gdk_window_set_icon_name(window->window, "Device Details");
	gtk_signal_connect(GTK_OBJECT(window), "delete_event",
		GTK_SIGNAL_FUNC(delete_event), this);

  vbox66 = gtk_vbox_new (FALSE, 4);
  gtk_widget_show (vbox66);
  gtk_container_add (GTK_CONTAINER (window), vbox66);
  gtk_container_set_border_width (GTK_CONTAINER (vbox66), 4);

  table4 = gtk_table_new (7, 3, FALSE);
  gtk_widget_show (table4);
  gtk_box_pack_start (GTK_BOX (vbox66), table4, FALSE, FALSE, 0);
  gtk_table_set_row_spacings (GTK_TABLE (table4), 2);
  gtk_table_set_col_spacings (GTK_TABLE (table4), 4);

	entry8 = gtk_entry_new_with_max_length (64);
  gtk_widget_show (entry8);
  gtk_table_attach (GTK_TABLE (table4), entry8, 1, 2, 0, 1,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);

  label17 = gtk_label_new ("Name");
  gtk_widget_show (label17);
  gtk_table_attach (GTK_TABLE (table4), label17, 0, 1, 0, 1,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_misc_set_alignment (GTK_MISC (label17), 1, 0.5);

  label18 = gtk_label_new ("Community");
  gtk_widget_show (label18);
  gtk_table_attach (GTK_TABLE (table4), label18, 0, 1, 2, 3,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_misc_set_alignment (GTK_MISC (label18), 1, 0.5);

  entry9 = gtk_entry_new ();
  gtk_widget_show (entry9);
  gtk_table_attach (GTK_TABLE (table4), entry9, 1, 2, 1, 2,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);

  entry10 = gtk_entry_new ();
  gtk_widget_show (entry10);
  gtk_table_attach (GTK_TABLE (table4), entry10, 1, 2, 2, 3,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);

  label19 = gtk_label_new ("IP Address");
  gtk_widget_show (label19);
  gtk_table_attach (GTK_TABLE (table4), label19, 0, 1, 1, 2,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_misc_set_alignment (GTK_MISC (label19), 1, 0.5);

  label20 = gtk_label_new ("Description");
  gtk_widget_show (label20);
  gtk_table_attach (GTK_TABLE (table4), label20, 0, 1, 3, 4,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_misc_set_alignment (GTK_MISC (label20), 1, 0.5);

  label21 = gtk_label_new ("Contact");
  gtk_widget_show (label21);
  gtk_table_attach (GTK_TABLE (table4), label21, 0, 1, 4, 5,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_misc_set_alignment (GTK_MISC (label21), 1, 0.5);

  label22 = gtk_label_new ("Location");
  gtk_widget_show (label22);
  gtk_table_attach (GTK_TABLE (table4), label22, 0, 1, 5, 6,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_misc_set_alignment (GTK_MISC (label22), 1, 0.5);

  entry11 = gtk_entry_new ();
  gtk_widget_show (entry11);
  gtk_table_attach (GTK_TABLE (table4), entry11, 1, 2, 3, 4,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);

  entry12 = gtk_entry_new ();
  gtk_widget_show (entry12);
  gtk_table_attach (GTK_TABLE (table4), entry12, 1, 2, 4, 5,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);

  entry13 = gtk_entry_new ();
  gtk_widget_show (entry13);
  gtk_table_attach (GTK_TABLE (table4), entry13, 1, 2, 5, 6,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);

  notebook1 = gtk_notebook_new ();
  gtk_widget_show (notebook1);
  gtk_table_attach (GTK_TABLE (table4), notebook1, 2, 3, 0, 5,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (GTK_FILL), 0, 0);
  GTK_WIDGET_UNSET_FLAGS (notebook1, GTK_CAN_FOCUS);
  gtk_notebook_set_show_tabs (GTK_NOTEBOOK (notebook1), FALSE);
  gtk_notebook_set_show_border (GTK_NOTEBOOK (notebook1), FALSE);

  pixmap2 = create_pixmap(window, (char**) lrouter_xpm);
  gtk_widget_show (pixmap2);
  gtk_container_add (GTK_CONTAINER (notebook1), pixmap2);

  pixmap3 = create_pixmap(window, (char**) lswitch_xpm);
  gtk_widget_show (pixmap2);
  gtk_container_add (GTK_CONTAINER (notebook1), pixmap3);

  pixmap4 = create_pixmap(window, (char**) lcomputer_xpm);
  gtk_widget_show (pixmap2);
  gtk_container_add (GTK_CONTAINER (notebook1), pixmap4);

  pixmap5 = create_pixmap(window, (char**) lserver_xpm);
  gtk_widget_show (pixmap2);
  gtk_container_add (GTK_CONTAINER (notebook1), pixmap5);

  combo2 = gtk_combo_new ();
  gtk_widget_show (combo2);
  gtk_table_attach (GTK_TABLE (table4), combo2, 2, 3, 5, 6,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
	gtk_signal_connect(GTK_OBJECT(GTK_COMBO(combo2)->list), "selection_changed",
		GTK_SIGNAL_FUNC (combo_cb), this);
  combo2_items = g_list_append (combo2_items, (gpointer) "Router");
  combo2_items = g_list_append (combo2_items, (gpointer) "Switch");
  combo2_items = g_list_append (combo2_items, (gpointer) "Bridge");
  combo2_items = g_list_append (combo2_items, (gpointer) "Hub");
  gtk_combo_set_popdown_strings (GTK_COMBO (combo2), combo2_items);
  g_list_free (combo2_items);

	entry14 = GTK_COMBO (combo2)->entry;
  gtk_widget_show (entry14);
  gtk_widget_set_usize (entry14, 100, -2);
  gtk_entry_set_editable (GTK_ENTRY (entry14), FALSE);
  gtk_entry_set_text (GTK_ENTRY (entry14), "Router");

  label23 = gtk_label_new ("Last Seen");
  gtk_widget_show (label23);
  gtk_table_attach (GTK_TABLE (table4), label23, 0, 1, 6, 7,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_misc_set_alignment (GTK_MISC (label23), 1, 0.5);

  label24 = gtk_label_new ("Wed 23rd Sep 2000 23:38:40");
  gtk_widget_show (label24);
  gtk_table_attach (GTK_TABLE (table4), label24, 1, 3, 6, 7,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 4, 2);
  gtk_misc_set_alignment (GTK_MISC (label24), 0, 0.5);

  frame14 = gtk_frame_new ("Interfaces");
  gtk_widget_show (frame14);
  gtk_box_pack_start (GTK_BOX (vbox66), frame14, TRUE, TRUE, 0);

  hbox85 = gtk_hbox_new (FALSE, 4);
  gtk_widget_show (hbox85);
  gtk_container_add (GTK_CONTAINER (frame14), hbox85);
  gtk_container_set_border_width (GTK_CONTAINER (hbox85), 4);

  scrolledwindow33 = gtk_scrolled_window_new (NULL, NULL);
  gtk_widget_show (scrolledwindow33);
  gtk_box_pack_start (GTK_BOX (hbox85), scrolledwindow33, TRUE, TRUE, 0);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow33), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);

  clist30 = gtk_clist_new (5);
  gtk_widget_show (clist30);
  gtk_container_add (GTK_CONTAINER (scrolledwindow33), clist30);
  gtk_clist_set_column_width (GTK_CLIST (clist30), 0, 42);
  gtk_clist_set_column_width (GTK_CLIST (clist30), 1, 64);
  gtk_clist_set_column_width (GTK_CLIST (clist30), 2, 92);
  gtk_clist_set_column_width (GTK_CLIST (clist30), 3, 56);
  gtk_clist_set_column_width (GTK_CLIST (clist30), 4, 50);
  gtk_clist_column_titles_show (GTK_CLIST (clist30));
	gtk_signal_connect(GTK_OBJECT(clist30), "select_row",
		GTK_SIGNAL_FUNC(select_row_cb), this);
	gtk_signal_connect(GTK_OBJECT(clist30), "unselect_row",
		GTK_SIGNAL_FUNC(unselect_row_cb), this);

  label175 = gtk_label_new ("Index");
  gtk_widget_show (label175);
  gtk_clist_set_column_widget (GTK_CLIST (clist30), 0, label175);

  label176 = gtk_label_new ("Port");
  gtk_widget_show (label176);
  gtk_clist_set_column_widget (GTK_CLIST (clist30), 1, label176);

  label177 = gtk_label_new ("MAC Address");
  gtk_widget_show (label177);
  gtk_clist_set_column_widget (GTK_CLIST (clist30), 2, label177);

  label178 = gtk_label_new ("Speed");
  gtk_widget_show (label178);
  gtk_clist_set_column_widget (GTK_CLIST (clist30), 3, label178);

  label179 = gtk_label_new ("Type");
  gtk_widget_show (label179);
  gtk_clist_set_column_widget (GTK_CLIST (clist30), 4, label179);

  scrolledwindow32 = gtk_scrolled_window_new (NULL, NULL);
  gtk_widget_show (scrolledwindow32);
  gtk_box_pack_start (GTK_BOX (hbox85), scrolledwindow32, FALSE, TRUE, 0);
  gtk_widget_set_usize (scrolledwindow32, 140, 120);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow32), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);

  clist29 = gtk_clist_new (1);
  gtk_widget_show (clist29);
  gtk_container_add (GTK_CONTAINER (scrolledwindow32), clist29);
  gtk_clist_set_column_width (GTK_CLIST (clist29), 0, 71);
  gtk_clist_column_titles_show (GTK_CLIST (clist29));

  label174 = gtk_label_new ("IP Addresses");
  gtk_widget_show (label174);
  gtk_clist_set_column_widget (GTK_CLIST (clist29), 0, label174);

  hbox87 = gtk_hbox_new (FALSE, 4);
  gtk_widget_show (hbox87);
  gtk_box_pack_start (GTK_BOX (vbox66), hbox87, FALSE, TRUE, 0);

  checkbutton15 = gtk_check_button_new_with_label ("Grab Graph Data");
  gtk_widget_show (checkbutton15);
  gtk_box_pack_end (GTK_BOX (hbox87), checkbutton15, FALSE, FALSE, 0);

  checkbutton14 = gtk_check_button_new_with_label ("Grab Topology Details");
  gtk_widget_show (checkbutton14);
  gtk_box_pack_end (GTK_BOX (hbox87), checkbutton14, FALSE, FALSE, 0);

  hseparator4 = gtk_hseparator_new ();
  gtk_widget_show (hseparator4);
  gtk_box_pack_start (GTK_BOX (vbox66), hseparator4, FALSE, TRUE, 0);

  hbox84 = gtk_hbox_new (TRUE, 4);
  gtk_widget_show (hbox84);
  gtk_box_pack_start (GTK_BOX (vbox66), hbox84, FALSE, TRUE, 0);

  button92 = gtk_button_new_with_label ("Accept");
  gtk_widget_show (button92);
  gtk_box_pack_start (GTK_BOX (hbox84), button92, FALSE, FALSE, 0);
  gtk_widget_set_usize (button92, 60, -2);
  gtk_signal_connect (GTK_OBJECT (button92), "clicked",
		GTK_SIGNAL_FUNC (ok_cb), this);

  button93 = gtk_button_new_with_label ("Cancel");
  gtk_widget_show (button93);
  gtk_box_pack_start (GTK_BOX (hbox84), button93, FALSE, FALSE, 0);
  gtk_widget_set_usize (button93, 60, -2);
  gtk_signal_connect (GTK_OBJECT (button93), "clicked",
		GTK_SIGNAL_FUNC (cancel_cb), this);
}

DeviceDetails::~DeviceDetails()
{
}

void DeviceDetails::empty()
{
	gtk_entry_set_text(GTK_ENTRY(entry8), "");
	gtk_entry_set_text(GTK_ENTRY(entry9), "");
	gtk_entry_set_text(GTK_ENTRY(entry10), "");
	gtk_entry_set_text(GTK_ENTRY(entry11), "");
	gtk_entry_set_text(GTK_ENTRY(entry12), "");
	gtk_entry_set_text(GTK_ENTRY(entry13), "");
	gtk_frame_set_label(GTK_FRAME(frame14), "Interfaces");
	gtk_label_set_text(GTK_LABEL(label24), "");
	gtk_clist_clear(GTK_CLIST(clist30));
	gtk_clist_clear(GTK_CLIST(clist29));
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (checkbutton14), FALSE);
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (checkbutton15), FALSE);
	
}

void DeviceDetails::show()
{
	gchar *data[5];
	char buf[256];
	Interface *in;
	Link *ln;
	int row;

	gtk_widget_show(window);
	gdk_window_raise(window->window);

	empty();

	if (device!=NULL)
	{
		gtk_entry_set_text(GTK_ENTRY(entry8), device->name);
		gtk_entry_set_text(GTK_ENTRY(entry9), device->pingname.getstring());
		gtk_entry_set_text(GTK_ENTRY(entry10), device->community);
		gtk_entry_set_text(GTK_ENTRY(entry11), device->descr);
		gtk_entry_set_text(GTK_ENTRY(entry13), device->location);
		gtk_entry_set_text(GTK_ENTRY(entry12), device->contact);
		sprintf(buf, "Interfaces (%d)", device->interfaces.size());
		gtk_frame_set_label(GTK_FRAME(frame14), buf);
		if (device->last_seen.zero()==0)
		{
			gtk_label_set_text(GTK_LABEL(label24), device->last_seen.date_to_str(1));
		}
		gtk_clist_freeze(GTK_CLIST(clist30));
		gtk_clist_clear(GTK_CLIST(clist30));

		row = 0;
		for (ln=device->interfaces.get_head(); ln!=NULL; ln=ln->next)
		{
			in = (Interface*) ln->item;
			sprintf(buf, "%d", in->index); data[0] = strdup(buf);
			data[1] = strdup(in->descr);
			data[2] = strdup(in->mac.getstring());
			speed_to_str(in->speed, buf); data[3] = strdup(buf);
			interface_type_to_str(in->type, buf); data[4] = strdup(buf);
			gtk_clist_append(GTK_CLIST(clist30), data);
			delete data[0]; delete data[1]; delete data[2];
			delete data[3]; delete data[4];
			sprintf(buf, "%d", in->index);
			if (in->status=='1')
				clist_pixtext(window, GTK_CLIST(clist30), row, 0, (gchar **)intup_xpm, buf);
			else
				clist_pixtext(window, GTK_CLIST(clist30), row, 0, (gchar **)intdown_xpm, buf);
			gtk_clist_set_row_data(GTK_CLIST(clist30), row, in);
			row++;
		}

		gtk_clist_thaw(GTK_CLIST(clist30));

		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (checkbutton14),
			(device->grab_top=='1')?TRUE:FALSE);
		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (checkbutton15),
			(device->grab_stats=='1')?TRUE:FALSE);
		gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(combo2)->entry), "Switch");
	}
}

void DeviceDetails::hide()
{
	gtk_widget_hide(window);
}

void DeviceDetails::save()
{
	if (device!=NULL)
	{
		strcpy(device->name, gtk_entry_get_text(GTK_ENTRY(entry8)));
		device->pingname.setstring(gtk_entry_get_text(GTK_ENTRY(entry9)));
		strcpy(device->community, gtk_entry_get_text(GTK_ENTRY(entry10)));
		strcpy(device->descr, gtk_entry_get_text(GTK_ENTRY(entry11)));
		strcpy(device->location, gtk_entry_get_text(GTK_ENTRY(entry13)));
		strcpy(device->contact, gtk_entry_get_text(GTK_ENTRY(entry12)));
		device->grab_top = (GTK_TOGGLE_BUTTON(checkbutton14)->active)?'1':'0';
		device->grab_stats = (GTK_TOGGLE_BUTTON(checkbutton15)->active)?'1':'0';
	}

	main->gui->update_lists();
}

void DeviceDetails::select_row(int row)
{
	Link *ln2;
	Interface *in;
	IPAddress *ip;
	char buf[256];
	gchar *data[1];

	in = (Interface*) gtk_clist_get_row_data(GTK_CLIST(clist30), row);

	gtk_clist_clear(GTK_CLIST(clist29));
	for (ln2=in->ip_addresses.get_head(); ln2!=NULL; ln2=ln2->next)
	{
		ip = (IPAddress*) ln2->item;
		strcpy(buf, ip->getstring());
		data[0] = strdup(buf);
		gtk_clist_append(GTK_CLIST(clist29), data);
		delete data[0];
	}
}

void DeviceDetails::unselect_row(int row)
{
	gtk_clist_clear(GTK_CLIST(clist29));
}

