/* features.cc
 * Copyright (C) 2001 The University of Newcastle upon Tyne
 *
 * Written by Paul Coates (Paul.Coates@ncl.ac.uk)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 */

#include <gtk/gtk.h>
#include <unistd.h>
#include <string.h>
#include <stdio.h>
#include "main.h"
#include "gui.h"
#include "features.h"
#include "licon.xpm"
#include "sessions.h"

static void on_Close_activate(GtkWidget *w, gpointer data)
{ ((Features*)data)->hide(); }

static gint delete_event(GtkWidget *widget, GdkEvent *event, gpointer data)
{ ((Features*)data)->hide(); return (TRUE); }

Features::Features(Main *m)
{
	GtkWidget *vbox1;
	GtkWidget *menubar1;
	GtkWidget *file1;
	GtkWidget *file1_menu;
	GtkWidget *close1;
	GtkWidget *notebook1;
	GtkWidget *label;
	GtkWidget *page;
	int i;

	main = m;

	window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_window_set_title (GTK_WINDOW (window), "Details");
  gtk_widget_realize (window);
	window_icon(window, licon_xpm);
	gdk_window_set_icon_name(window->window, "Settings");
	gtk_signal_connect (GTK_OBJECT (window), "delete_event",
		GTK_SIGNAL_FUNC(delete_event), this);

	vbox1 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox1);
  gtk_container_add (GTK_CONTAINER (window), vbox1);

  menubar1 = gtk_menu_bar_new ();
  gtk_widget_show (menubar1);
  gtk_box_pack_start (GTK_BOX (vbox1), menubar1, FALSE, FALSE, 0);

  file1 = gtk_menu_item_new_with_label ("File");
  gtk_widget_show (file1);
  gtk_container_add (GTK_CONTAINER (menubar1), file1);

  file1_menu = gtk_menu_new ();
  gtk_menu_item_set_submenu (GTK_MENU_ITEM (file1), file1_menu);

	close1 = gtk_menu_item_new_with_label ("Close");
  gtk_widget_show (close1);
  gtk_container_add (GTK_CONTAINER (file1_menu), close1);
  gtk_signal_connect(GTK_OBJECT(close1), "activate",
                GTK_SIGNAL_FUNC(on_Close_activate), this);

  notebook1 = gtk_notebook_new ();
  gtk_widget_show (notebook1);
  gtk_box_pack_start (GTK_BOX (vbox1), notebook1, TRUE, TRUE, 0);

	sessions = new Sessions(main);

  gtk_container_add (GTK_CONTAINER (notebook1), sessions->window);

	char *labels[] = { "Session" };

	for (i=0; i<1; i++)
	{
		page = gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook1), i);
  	label = gtk_label_new (labels[i]);
		gtk_widget_show (label);
  	gtk_notebook_set_tab_label (GTK_NOTEBOOK (notebook1), page, label);
	}
	// also filters and views
}

Features::~Features()
{
	if (sessions!=NULL) delete sessions;
}

void Features::show()
{
	gtk_widget_show(window);
	gdk_window_raise(window->window);
}

void Features::hide()
{
	gtk_widget_hide(window);
}

