/* filterdetails.cc
 * Copyright (C) 2001 The University of Newcastle upon Tyne
 *
 * Written by Paul Coates (Paul.Coates@ncl.ac.uk)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 */

#include <gtk/gtk.h>
#include <unistd.h>
#include <string.h>
#include <stdio.h>
#include <pwd.h>
#include "main.h"
#include "monitor.h"
#include "filtersidebar.h"
#include "filterdetails.h"
#include "filtermodule.h"
#include "gui.h"
#include "licon.xpm"
#include "session.h"

/***************************************************************************
 Callbacks
 ***************************************************************************/

static void accept_cb(GtkWidget *w, gpointer data)
{ ((FilterDetails*) data)->accept(); }
static void cancel_cb(GtkWidget *w, gpointer data)
{ ((FilterDetails*) data)->hide(); }
static void ok_cb(GtkWidget *w, gpointer data)
{ ((FilterDetails*) data)->msg.hide(); }

/***************************************************************************
 FilterDetails Class
 ***************************************************************************/

static gint delete_event(GtkWidget *widget, GdkEvent *event, gpointer data)
{
	((FilterDetails*)data)->hide();
	return TRUE;
}

FilterDetails::FilterDetails(Main *m)
{
	main = m;
	orig = NULL;

	window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_window_set_modal (GTK_WINDOW (window), TRUE);
	gtk_window_set_title (GTK_WINDOW (window), "Filter Details");
	gtk_widget_realize(window);
	window_icon(window, licon_xpm);
	gdk_window_set_icon_name(window->window, "Filter Details");
	gtk_signal_connect(GTK_OBJECT(window), "delete_event",
		GTK_SIGNAL_FUNC(delete_event), this);

	vbox54 = gtk_vbox_new (FALSE, 4);
	gtk_widget_show (vbox54);
	gtk_container_add (GTK_CONTAINER (window), vbox54);
	gtk_container_set_border_width (GTK_CONTAINER (vbox54), 4);

	hbox75 = gtk_hbox_new (FALSE, 4);
	gtk_widget_show (hbox75);
	gtk_box_pack_start (GTK_BOX (vbox54), hbox75, FALSE, TRUE, 0);

	label146 = gtk_label_new ("Filter Name");
	gtk_widget_show (label146);
	gtk_box_pack_start (GTK_BOX (hbox75), label146, FALSE, FALSE, 0);

	entry33 = gtk_entry_new ();
	gtk_widget_show (entry33);
	gtk_box_pack_start (GTK_BOX (hbox75), entry33, FALSE, TRUE, 0);

	filtermodule = new FilterModule(window);
	gtk_box_pack_start (GTK_BOX (vbox54), filtermodule->vbox58, TRUE, TRUE, 0);

	hseparator3 = gtk_hseparator_new ();
	gtk_widget_show (hseparator3);
	gtk_box_pack_start (GTK_BOX (vbox54), hseparator3, FALSE, TRUE, 0);

	hbox76 = gtk_hbox_new (TRUE, 4);
	gtk_widget_show (hbox76);
	gtk_box_pack_start (GTK_BOX (vbox54), hbox76, FALSE, TRUE, 0);

	button71 = gtk_button_new_with_label ("Cancel");
	gtk_widget_show (button71);
	gtk_box_pack_end (GTK_BOX (hbox76), button71, FALSE, FALSE, 0);
	gtk_widget_set_usize (button71, 60, -2);
  gtk_signal_connect (GTK_OBJECT (button71), "clicked",
		GTK_SIGNAL_FUNC (cancel_cb), this);

	button70 = gtk_button_new_with_label ("Accept");
	gtk_widget_show (button70);
	gtk_box_pack_end (GTK_BOX (hbox76), button70, FALSE, FALSE, 0);
	gtk_widget_set_usize (button70, 60, -2);
  gtk_signal_connect (GTK_OBJECT (button70), "clicked",
		GTK_SIGNAL_FUNC (accept_cb), this);

	msg.setTitle("Filter");
	msg.setLabel("");
	msg.setButtons(3);
	gtk_signal_connect (GTK_OBJECT (msg.button_yes), "clicked",
		GTK_SIGNAL_FUNC (ok_cb), this);
}

FilterDetails::~FilterDetails()
{
	if (filtermodule!=NULL) delete filtermodule;
}

void FilterDetails::show()
{
	gtk_widget_show(window);
	gdk_window_raise(window->window);
//	gtk_grab_add(window);
}

void FilterDetails::hide()
{
	gtk_widget_hide(window);
}

void FilterDetails::copytowidgets(Filter *flt)
{
	gtk_entry_set_text(GTK_ENTRY(entry33), flt->name);
	filtermodule->copytowidgets(flt);
}

void FilterDetails::copyfromwidgets(Filter *flt)
{
	flt->empty();
	strcpy(flt->name, gtk_entry_get_text(GTK_ENTRY(entry33)));
	filtermodule->copyfromwidgets(flt);
}

void FilterDetails::accept()
{
	Filter *f;
	char name[256];

	strcpy(name, gtk_entry_get_text(GTK_ENTRY(entry33)));

	if (strcmp(name, "")==0)
	{
		msg.setLabel("You must enter a filter name");
		msg.show();
	}else
	if (main->session->uniquefiltername(orig, name)==0)
	{
		msg.setLabel("You must enter a unique filter name");
		msg.show();
	}else
	{
		if (orig==NULL)
		{
			f = new Filter();
			copyfromwidgets(f);
			main->session->filters.tag(f);
		}else
			copyfromwidgets(orig);
		main->session->dirty='y';
		main->setTitle();
		main->gui->monitor->filtersidebar->update();
		orig = NULL;
		hide();
	}
}

