/* gui.cc
 * Copyright (C) 2001 The University of Newcastle upon Tyne
 *
 * Written by Paul Coates (Paul.Coates@ncl.ac.uk)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 */

#include <gtk/gtk.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <stdio.h>
#include <pwd.h>
#include "main.h"
#include "data.h"
#include "gui.h"
#include "discover.h"
#include "devicesidebar.h"
#include "summarysidebar.h"
#include "filtersidebar.h"
#include "viewsidebar.h"
#include "monitor.h"
#include "diagram.h"
#include "stats.h"
#include "print.h"
#include "subnets.h"
#include "message.h"
#include "filterdetails.h"
#include "preferences.h"
#include "session.h"
#include "devicedetails.h"
#include "machinedetails.h"

Gui::Gui(Main *m)
{
	main = m;
	preferences = new Preferences(main);
	monitor = new Monitor(main);
	discover = new Discover(main);
	stats.empty();
	print = new Print(main);
	subnets = new Subnets(main);
	message = new Message();
	filterdetails = new FilterDetails(main);
	devicedetails = new DeviceDetails(main);
	machinedetails = new MachineDetails(main);
}

Gui::~Gui()
{
	if (preferences!=NULL) delete preferences;
	if (monitor!=NULL) delete monitor;
	if (discover!=NULL) delete discover;
	if (print!=NULL) delete print;
	if (subnets!=NULL) delete subnets;
	if (filterdetails!=NULL) delete filterdetails;
	if (devicedetails!=NULL) delete devicedetails;
	if (machinedetails!=NULL) delete machinedetails;
}

void Gui::update_views()
{
	main->session->data->position_links();
	main->session->data->apply_group();
	main->session->data->group_machines();
	monitor->viewer->draw();
}

void Gui::update_lists()
{
	monitor->devicesidebar->update();
	monitor->summarysidebar->update();
	monitor->filtersidebar->update();
	monitor->viewsidebar->update();
	subnets->refresh();
}

void Gui::graph_window()
{
	Stats *s;

	if (monitor->viewer->cur != NULL)
	{
		if ((monitor->viewer->cur->type == 'r') ||
				(monitor->viewer->cur->type == 's'))
		{
			s = new Stats(main);
			gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(s->combo3)->entry),
				((Device*)monitor->viewer->cur)->name);
			s->device_list();
			s->show();
			stats.tag(s);
		}else
		if (monitor->viewer->cur->type == 'n')
		{
			s = new Stats(main);
			s->show();
//			s->select_interface((Network*)monitor->viewer->cur);
			stats.tag(s);
		}
	}
}

void Gui::update_preferences()
{
	// call update method in each window that has preferences

/*
	Link *ln;
	GtkWidget *w;
	gchar *data[1];

	// PRINT_COMMAND
  gtk_entry_set_text (GTK_ENTRY (print->entry5), pref->print_command);

	// PRINT_ORIENTATION
	print->orientation = pref->print_orientation;
	if (pref->print_orientation==0)
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(print->radiobutton20),TRUE);
	else
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(print->radiobutton19),TRUE);


	// SUBNET_ENTRY
*/
}

/*****************************************************************************
 GUI Functions
 *****************************************************************************/

void window_icon(GtkWidget *widget, char **filename)
{
	GdkPixmap *gdkpixmap;
	GdkBitmap *mask;
  GtkWidget *pixmap;

	gdkpixmap = gdk_pixmap_create_from_xpm_d(widget->window, &mask,
  	&widget->style->bg[GTK_STATE_NORMAL], (gchar **)filename);
  pixmap = gtk_pixmap_new(gdkpixmap, mask);
	gtk_widget_show(pixmap);
	gdk_window_set_icon(widget->window, pixmap->window, gdkpixmap, mask);
  gdk_pixmap_unref(gdkpixmap);
  gdk_bitmap_unref(mask);
}

GtkWidget *menu_icon(GtkWidget *widget, char **pix, char *label, GtkAccelGroup *grp)
{
	GtkWidget *new2;
	GtkWidget *hbox1;
	GtkWidget *label1;
	GtkWidget *pix2;
  guint key;

  new2 = gtk_menu_item_new ();

  hbox1 = gtk_hbox_new (FALSE, 2);
  gtk_widget_show (hbox1);
  gtk_container_add (GTK_CONTAINER (new2), hbox1);

	pix2 = create_pixmap(widget, pix);
  gtk_box_pack_start (GTK_BOX (hbox1), pix2, FALSE, TRUE, 0);

  label1 = gtk_label_new (label);
  gtk_widget_show (label1);
  gtk_box_pack_start (GTK_BOX (hbox1), label1, FALSE, FALSE, 0);

	if (grp!=NULL)
	{
		key = gtk_label_parse_uline (GTK_LABEL (label1), label);
		gtk_widget_add_accelerator (new2, "activate_item", grp,
			key, GDK_CONTROL_MASK, (GtkAccelFlags) 0);
	}

	return new2;
}

GtkWidget *create_pixmap(GtkWidget *widget, char **filename)
{
	GdkPixmap *gdkpixmap;
	GdkBitmap *mask;
  GtkWidget *pixmap;

	gdkpixmap = gdk_pixmap_create_from_xpm_d(widget->window, &mask,
  	&widget->style->bg[GTK_STATE_NORMAL], (gchar **)filename);
  if (gdkpixmap == NULL)
    return (GtkWidget*)NULL;
  pixmap = gtk_pixmap_new(gdkpixmap, mask);
	gtk_widget_show(pixmap);
  gdk_pixmap_unref(gdkpixmap);
  gdk_bitmap_unref(mask);

  return pixmap;
}

GdkPixmap *load_pixmap(GtkWidget *widget, char *filename)
{
	GdkPixmap *gdkpixmap;
	GdkBitmap *mask;

	gdkpixmap = gdk_pixmap_create_from_xpm(widget->window, &mask,
  	&widget->style->bg[GTK_STATE_NORMAL], (gchar *)filename);

  return gdkpixmap;
}

void set_stipple(GtkWidget *widget, GdkGC *gc, char **filename)
{
	GdkPixmap *gdkpixmap;
	GdkBitmap *mask;

	gdkpixmap = gdk_pixmap_create_from_xpm_d(widget->window, &mask,
  	&widget->style->bg[GTK_STATE_NORMAL], (gchar **)filename);
	gdk_gc_set_stipple(gc, gdkpixmap);
}

void clist_pixmap(GtkWidget *widget, GtkCList *clist, gint row, gint column, char **filename)
{
	GdkPixmap *gdkpixmap;
	GdkBitmap *mask;

	gdkpixmap = gdk_pixmap_create_from_xpm_d(widget->window, &mask,
  	&widget->style->bg[GTK_STATE_NORMAL], (gchar **)filename);
  if (gdkpixmap == NULL) return;

	gtk_clist_set_pixmap(clist, row, column, gdkpixmap, mask);

  gdk_pixmap_unref(gdkpixmap);
  gdk_bitmap_unref(mask);
}

void clist_pixtext(GtkWidget *widget, GtkCList *clist, gint row, gint column, char **filename, gchar *str)
{
	GdkPixmap *gdkpixmap;
	GdkBitmap *mask;

	gdkpixmap = gdk_pixmap_create_from_xpm_d(widget->window, &mask,
  	&widget->style->bg[GTK_STATE_NORMAL], (gchar **)filename);
  if (gdkpixmap == NULL) return;

	gtk_clist_set_pixtext(clist, row, column, str, 4, gdkpixmap, mask);

  gdk_pixmap_unref(gdkpixmap);
  gdk_bitmap_unref(mask);
}

void alloc_color(GdkColor &col, guint16 r, guint16 g, guint16 b)
{
	GdkColormap *colormap = gdk_colormap_get_system();

	col.red   = r;
	col.green = g;
	col.blue  = b;
	gdk_color_alloc(colormap, &col);
}

void speed_to_str(unsigned long spd, char *buf)
{
	if (spd == 0)
		sprintf(buf, "0");
	else
	if ((spd%1000000000) == 0)
		sprintf(buf, "%luGb", (unsigned long) (spd/1000000000));
	else
	if ((spd%1000000) == 0)
		sprintf(buf, "%luMb", (unsigned long) (spd/1000000));
	else
	if ((spd%1000) == 0)
		sprintf(buf, "%lukb", (unsigned long) (spd/1000));
	else
		sprintf(buf, "%lu", spd);
}

void seconds_to_time_string(unsigned long sec, char *buf)
{
	if (sec < 300)
		sprintf(buf, "%lu seconds", sec);
	else
	if ((sec%60)==0)
		sprintf(buf, "%lu minutes", (sec/60));
}

void interface_type_to_str(int type, char *buf)
{
	char tmp[256];

	switch (type)
	{
		case 1:		sprintf(tmp, "Other"); break;
		case 6: 	sprintf(tmp, "Ethernet"); break;
		case 15:	sprintf(tmp, "FDDI"); break;
		case 18:	sprintf(tmp, "E1"); break;
		case 22:	sprintf(tmp, "Serial"); break;
		case 23:	sprintf(tmp, "PPP Serial"); break;
		case 28:	sprintf(tmp, "SLIP"); break;
		case 37:	sprintf(tmp, "ATM Layer"); break;
		case 39:	sprintf(tmp, "SONET / SDH"); break;
		case 49:	sprintf(tmp, "AAL5 Layer"); break;
//		case 53:	sprintf(tmp, ""); break;
		case 59:	sprintf(tmp, "LEC"); break;
		case 24:	sprintf(tmp, "Loopback"); break;
		default:	sprintf(tmp, "Unknown");
	}
	sprintf(buf, "%s (%d)", tmp, type);
}

