/* monitor.cc
 * Copyright (C) 2001 The University of Newcastle upon Tyne
 *
 * Written by Paul Coates (Paul.Coates@ncl.ac.uk)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 */

#include <gtk/gtk.h>
#include <gnome.h>
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include "monitor.h"
#include "discover.h"
#include "diagram.h"
#include "preferences.h"
#include "sessionlist.h"
#include "message.h"
#include "features.h"
#include "main.h"
#include "gui.h"
#include "data.h"
#include "print.h"
#include "stats.h"
#include "device.h"
#include "devicesidebar.h"
#include "searchsidebar.h"
#include "summarysidebar.h"
#include "viewsidebar.h"
#include "filtersidebar.h"
#include "session.h"
#include "subnets.h"

#include "licon.xpm"
#include "ssummary.xpm"
#include "srouter.xpm"
#include "ssearch.xpm"
#include "sview.xpm"

const gchar *authors[] = { "Paul Coates <Paul.Coates@ncl.ac.uk>", NULL };

static gint delete_event(GtkWidget *widget, GdkEvent *event, gpointer data)
{
	((Monitor*)data)->main->quit();
	return (TRUE);
}

// File Callbacks

static void new_cb(GtkWidget *w, gpointer data)
{ ((Monitor*)data)->main->session->new_session(1); }
static void open_cb(GtkWidget *w, gpointer data)
{ ((Monitor*)data)->main->session->new_session(2); }
static void save_cb(GtkWidget *w, gpointer data)
{ ((Monitor*)data)->main->session->have_title(); }
static void saveas_cb(GtkWidget *w, gpointer data)
{
	Monitor *m = (Monitor*) data;
	m->main->session->save_session_list.init(m->main->session->datadir);
	m->main->session->save_session_list.show();
}
static void print_cb(GtkWidget *w, gpointer data)
{}
static void exit_cb(GtkWidget *w, gpointer data)
{ ((Monitor*)data)->main->session->new_session(3);
	((Monitor*)data)->main->quit(); }


// Settings Callbacks

static void sidepanel_cb(GtkWidget *w, gpointer data)
{ ((Monitor*)data)->sidepanel(); }
static void labels_cb(GtkWidget *w, gpointer data)
{ ((Monitor*)data)->labels(); }
static void colour_cb(GtkWidget *w, gpointer data)
{ ((Monitor*)data)->colour(); }
static void default_cb(GtkWidget *w, gpointer data)
{
	Monitor *m = (Monitor*) data;
	gtk_widget_hide(m->opengl->mainmenu);
	gtk_widget_show(m->viewer->mainmenu);
	gtk_notebook_set_page(GTK_NOTEBOOK (m->notebook2), 0);
}
static void opengl_cb(GtkWidget *w, gpointer data)
{
	Monitor *m = (Monitor*) data;
	gtk_widget_hide(m->viewer->mainmenu);
	gtk_widget_show(m->opengl->mainmenu);
	gtk_notebook_set_page(GTK_NOTEBOOK (m->notebook2), 1);
}
static void preferences_cb(GtkWidget *w, gpointer data)
{ ((Monitor*)data)->main->gui->preferences->show(); }


// Help Callbacks

static void homepage_cb(GtkWidget *w, gpointer data)
{ gnome_url_show("http://netmon.ncl.ac.uk/"); }
static void manual_cb(GtkWidget *w, gpointer data)
{ gnome_help_goto(NULL, "ghelp:nomad"); }







static void about_cb(GtkWidget *w, gpointer data)
{ 
	static GtkWidget *dialog = NULL;
	GtkWidget *app;

	app = ((Monitor*)data)->mainwindow;

	if (dialog != NULL)
	{
		g_assert(GTK_WIDGET_REALIZED(dialog));
		gdk_window_show(dialog->window);
		gdk_window_raise(dialog->window);
	}else
	{
		gchar *fname = user_file("logo.png");
		dialog = gnome_about_new(_("Nomad"), "0.2.4",
			"Copyright (C) 2001 Newcastle University",
			authors, _("Network Mapping & Monitoring Tool"), fname);
		g_free(fname);

		gtk_signal_connect(GTK_OBJECT(dialog), "destroy",
			GTK_SIGNAL_FUNC(gtk_widget_destroyed), &dialog);
		gnome_dialog_set_parent(GNOME_DIALOG(dialog), GTK_WINDOW(app));
		gtk_widget_show(dialog);
	}
}

static GnomeUIInfo file1_menu_uiinfo[] =
{
  GNOMEUIINFO_MENU_NEW_ITEM (N_("_New"), NULL, new_cb, NULL),
  GNOMEUIINFO_MENU_OPEN_ITEM (open_cb, NULL),
  GNOMEUIINFO_MENU_SAVE_ITEM (save_cb, NULL),
  GNOMEUIINFO_MENU_SAVE_AS_ITEM (saveas_cb, NULL),
  GNOMEUIINFO_SEPARATOR,
  GNOMEUIINFO_MENU_PRINT_ITEM (print_cb, NULL),
  GNOMEUIINFO_SEPARATOR,
  GNOMEUIINFO_MENU_EXIT_ITEM (exit_cb, NULL),
  GNOMEUIINFO_END
};

static GnomeUIInfo default1_uiinfo[] =
{
  {
    GNOME_APP_UI_ITEM, N_("Default"), NULL,
    (gpointer) default_cb, NULL, NULL,
    GNOME_APP_PIXMAP_NONE, NULL,
    0, (GdkModifierType) 0, NULL
  },
  {
    GNOME_APP_UI_ITEM, N_("OpenGL"), NULL,
    (gpointer) opengl_cb, NULL, NULL,
    GNOME_APP_PIXMAP_NONE, NULL,
    0, (GdkModifierType) 0, NULL
  },
  GNOMEUIINFO_END
};

static GnomeUIInfo viewer1_menu_uiinfo[] =
{
  {
    GNOME_APP_UI_RADIOITEMS, NULL, NULL, default1_uiinfo,
    NULL, NULL, GNOME_APP_PIXMAP_NONE, NULL, 0,
    (GdkModifierType) 0, NULL
  },
  GNOMEUIINFO_END
};

static GnomeUIInfo settings1_menu_uiinfo[] =
{
  {
    GNOME_APP_UI_TOGGLEITEM, N_("Side Panel"), NULL,
    (gpointer) sidepanel_cb, NULL, NULL,
    GNOME_APP_PIXMAP_NONE, NULL,
    0, (GdkModifierType) 0, NULL
  },
  {
    GNOME_APP_UI_TOGGLEITEM, N_("Labels"), NULL,
    (gpointer) labels_cb, NULL, NULL,
    GNOME_APP_PIXMAP_NONE, NULL,
    0, (GdkModifierType) 0, NULL
  },
  {
    GNOME_APP_UI_TOGGLEITEM, N_("Coloured Links"), NULL,
    (gpointer) colour_cb, NULL, NULL,
    GNOME_APP_PIXMAP_NONE, NULL,
    0, (GdkModifierType) 0, NULL
  },
  {
    GNOME_APP_UI_SUBTREE, N_("Viewer"), NULL,
    viewer1_menu_uiinfo, NULL, NULL,
    GNOME_APP_PIXMAP_NONE, NULL,
    0, (GdkModifierType) 0, NULL
  },
  GNOMEUIINFO_SEPARATOR,
  GNOMEUIINFO_MENU_PREFERENCES_ITEM (preferences_cb, NULL),
  GNOMEUIINFO_END
};

static GnomeUIInfo help1_menu_uiinfo[] =
{
	{
		GNOME_APP_UI_ITEM, N_("Homepage"), NULL,
		(gpointer) homepage_cb, NULL, NULL,
		GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_HOME,
		0, (GdkModifierType) 0, NULL
	},
	{
		GNOME_APP_UI_ITEM, N_("Manual"), NULL,
		(gpointer) manual_cb, NULL, NULL,
		GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_BOOK_OPEN,
		0, (GdkModifierType) 0, NULL
	},
	GNOMEUIINFO_MENU_ABOUT_ITEM (about_cb, NULL),
	GNOMEUIINFO_END
};

static GnomeUIInfo menubar1_uiinfo[] =
{
  GNOMEUIINFO_MENU_FILE_TREE (file1_menu_uiinfo),
  GNOMEUIINFO_MENU_SETTINGS_TREE (settings1_menu_uiinfo),
  GNOMEUIINFO_MENU_HELP_TREE (help1_menu_uiinfo),
  GNOMEUIINFO_END
};



/*
static void subnets_cb(GtkWidget *w, gpointer data)
{
	Monitor* m = (Monitor*) data;
	m->main->gui->subnets->show();
}

static void discover_cb(GtkWidget *w, gpointer data)
{ ((Monitor*)data)->main->gui->discover->show(); }
*/


/***************************************************************************
 Monitor Class
 ***************************************************************************/

Monitor::Monitor(Main *m)
{
	GtkWidget *vbox77;
	GtkWidget *hseparator1;
	gchar *str;

	main = m;

	// set default preferences
	side_panel = '1';
	which_viewer = '0';
	str = gnome_config_get_string("/nomad/settings/sidepanel=1");
	if (str!=NULL) side_panel = str[0]; g_free(str);
	str = gnome_config_get_string("/nomad/settings/viewer=0");
	if (str!=NULL) which_viewer = str[0]; g_free(str);

  mainwindow = gtk_window_new(GTK_WINDOW_TOPLEVEL);
  gtk_window_set_title(GTK_WINDOW (mainwindow), "Nomad");
	gtk_widget_set_usize(mainwindow, 800, 600);
	gtk_widget_realize(mainwindow);
	window_icon(mainwindow, licon_xpm);
	gdk_window_set_icon_name(mainwindow->window, "Nomad");
	gtk_signal_connect(GTK_OBJECT(mainwindow), "delete_event",
		GTK_SIGNAL_FUNC(delete_event), this);
	gtk_signal_connect(GTK_OBJECT(mainwindow), "destroy",
		GTK_SIGNAL_FUNC(delete_event), this);

  pixmap[0] = create_pixmap(mainwindow, ssummary_xpm);
  pixmap[1] = create_pixmap(mainwindow, srouter_xpm);
  pixmap[2] = create_pixmap(mainwindow, ssearch_xpm);
  pixmap[3] = create_pixmap(mainwindow, sview_xpm);

	viewer = new Viewer(mainwindow, main);
	opengl = new OpenGL(mainwindow, main);
	devicesidebar = new DeviceSideBar(main);
	searchsidebar = new SearchSideBar(main);
	summarysidebar = new SummarySideBar(main);
	viewsidebar = new ViewSideBar(main);
	filtersidebar = new FilterSideBar(main);

  vbox1 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox1);
  gtk_container_add (GTK_CONTAINER (mainwindow), vbox1);

  menubar = gtk_menu_bar_new();
  gtk_widget_show(menubar);
  gtk_box_pack_start(GTK_BOX(vbox1), menubar, FALSE, FALSE, 0);
  gnome_app_fill_menu_with_data (GTK_MENU_SHELL (menubar), menubar1_uiinfo,
		NULL, FALSE, 0, this);

	hpaned1 = gtk_hpaned_new ();
  gtk_widget_show (hpaned1);
  gtk_box_pack_start (GTK_BOX (vbox1), hpaned1, TRUE, TRUE, 0);
  gtk_paned_set_gutter_size (GTK_PANED (hpaned1), 10);
	gtk_paned_set_handle_size (GTK_PANED (hpaned1), 10);
  gtk_paned_set_position (GTK_PANED (hpaned1), 160);

  notebook1 = gtk_notebook_new ();
  gtk_widget_show (notebook1);
  gtk_paned_pack1 (GTK_PANED (hpaned1), notebook1, FALSE, TRUE);

  gtk_container_add (GTK_CONTAINER (notebook1), summarysidebar->vbox2);
  gtk_container_add (GTK_CONTAINER (notebook1), devicesidebar->vbox67);
  gtk_container_add (GTK_CONTAINER (notebook1), searchsidebar->vbox12);

  vbox77 = gtk_vbox_new (FALSE, 4);
  gtk_widget_show (vbox77);
  gtk_container_add (GTK_CONTAINER (notebook1), vbox77);

	gtk_box_pack_start (GTK_BOX (vbox77), viewsidebar->vbox67, TRUE, TRUE, 0);
  hseparator1 = gtk_hseparator_new ();
  gtk_widget_show (hseparator1);
  gtk_box_pack_start (GTK_BOX (vbox77), hseparator1, FALSE, TRUE, 2);
	gtk_box_pack_start (GTK_BOX (vbox77), filtersidebar->vbox56, TRUE, TRUE, 0);

	gtk_notebook_set_tab_label (GTK_NOTEBOOK (notebook1),
		gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook1), 0), pixmap[0]);
  gtk_notebook_set_tab_label (GTK_NOTEBOOK (notebook1),
		gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook1), 1), pixmap[1]);
  gtk_notebook_set_tab_label (GTK_NOTEBOOK (notebook1),
		gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook1), 2), pixmap[2]);
  gtk_notebook_set_tab_label (GTK_NOTEBOOK (notebook1),
		gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook1), 3), pixmap[3]);

	notebook2 = gtk_notebook_new ();
  gtk_widget_show (notebook2);
  gtk_paned_pack2 (GTK_PANED (hpaned1), notebook2, TRUE, FALSE);
	gtk_notebook_set_show_tabs(GTK_NOTEBOOK (notebook2), FALSE);
	gtk_notebook_set_show_border(GTK_NOTEBOOK (notebook2), FALSE);

  gtk_container_add (GTK_CONTAINER (notebook2), viewer->vbox3);
  gtk_container_add (GTK_CONTAINER (notebook2), opengl->vbox3);

	gtk_menu_bar_insert(GTK_MENU_BAR(menubar), viewer->mainmenu, 1);
	gtk_menu_bar_insert(GTK_MENU_BAR(menubar), opengl->mainmenu, 1);


// Apply Preferences

	gtk_check_menu_item_set_active(
		GTK_CHECK_MENU_ITEM(settings1_menu_uiinfo[0].widget),
		(side_panel=='1')?TRUE:FALSE);

	gtk_check_menu_item_set_active(
		GTK_CHECK_MENU_ITEM(settings1_menu_uiinfo[1].widget),
		(viewer->diagram->showlabels=='1')?TRUE:FALSE);

	gtk_check_menu_item_set_active(
		GTK_CHECK_MENU_ITEM(settings1_menu_uiinfo[2].widget),
		(viewer->diagram->showcolourlines=='1')?TRUE:FALSE);

	if (which_viewer=='0')
	{
		gtk_check_menu_item_set_active(
			GTK_CHECK_MENU_ITEM(default1_uiinfo[0].widget), TRUE);
		gtk_widget_hide(opengl->mainmenu);
	}else
	if (which_viewer=='1')
	{
		gtk_check_menu_item_set_active(
			GTK_CHECK_MENU_ITEM(default1_uiinfo[1].widget), TRUE);
		gtk_widget_hide(viewer->mainmenu);
	}
}

Monitor::~Monitor()
{
	if (viewer!=NULL) delete viewer;
	if (summarysidebar!=NULL) delete summarysidebar;
	if (devicesidebar!=NULL) delete devicesidebar;
	if (searchsidebar!=NULL) delete searchsidebar;
	if (viewsidebar!=NULL) delete viewsidebar;
}

void Monitor::show()
{
	summarysidebar->init();
	gtk_widget_show(mainwindow);
	gdk_window_raise(mainwindow->window);
}

void Monitor::hide()
{
	gtk_widget_hide(mainwindow);
}

void Monitor::sidepanel()
{
	if (GTK_CHECK_MENU_ITEM(settings1_menu_uiinfo[0].widget)->active)
	{
		gtk_widget_show(hpaned1);
		gtk_widget_ref(notebook2);
		if (notebook2->parent == vbox1)
			gtk_container_remove(GTK_CONTAINER(vbox1), notebook2);
		if (notebook2->parent == NULL)
			gtk_container_add(GTK_CONTAINER(hpaned1), notebook2);
		gtk_widget_unref(notebook2);
	}else
	{
		gtk_widget_hide(hpaned1);
		gtk_widget_ref(notebook2);
		if (notebook2->parent == hpaned1)
			gtk_container_remove(GTK_CONTAINER(hpaned1), notebook2);
		if (notebook2->parent == NULL)
			gtk_container_add(GTK_CONTAINER(vbox1), notebook2);
		gtk_widget_unref(notebook2);
	}
}

void Monitor::labels()
{
	viewer->diagram->showlabels =
		(GTK_CHECK_MENU_ITEM(settings1_menu_uiinfo[1].widget)->active)?'1':'0';
	viewer->draw();
}

void Monitor::colour()
{
	viewer->diagram->showcolourlines =
		(GTK_CHECK_MENU_ITEM(settings1_menu_uiinfo[2].widget)->active)?'1':'0';
	viewer->draw();
}

