/* preferences.cc
 * Copyright (C) 2001 The University of Newcastle upon Tyne
 *
 * Written by Paul Coates (Paul.Coates@ncl.ac.uk)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 */

#include <glade/glade-xml.h>
#include <gnome.h>
#include <gtk/gtkinvisible.h>
#include <gtk/gtk.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <stdio.h>
#include "preferences.h"
#include "main.h"
#include "session.h"
#include "gui.h"
#include "graphio.h"
#include "header.h"
#include "licon.xpm"
#include "gnome-clock.xpm"
#include "gnome-stripchart.xpm"
#include "gnome-session.xpm"
#include "helix-setup-users3.xpm"
#include "faq-48.xpm"

static void select_community_cb(GtkWidget *clist, gint row, gint col,
	GdkEventButton *e, gpointer data)
{ ((Preferences*)data)->select_community(row); }
static void unselect_community_cb(GtkWidget *clist, gint row, gint col,
	GdkEventButton *e, gpointer data)
{ ((Preferences*)data)->unselect_community(); }
static void add_community_cb(GtkWidget *w, gpointer data)
{ ((Preferences*)data)->add_community(); }
static void del_community_cb(GtkWidget *w, gpointer data)
{ ((Preferences*)data)->del_community(); }
static void accept_cb(GtkWidget *w, gpointer data)
{ ((Preferences*)data)->save(); ((Preferences*)data)->hide(); }
static void cancel_cb(GtkWidget *w, gpointer data)
{ ((Preferences*)data)->load(); ((Preferences*)data)->hide(); }
static void sidebar_button_pressed_cb(GtkWidget *w, gpointer data)
{ ((Preferences*)data)->sidebar_button_pressed(w); }
static void sidebar_icon_pressed_cb(GtkWidget *w, gpointer data)
{ ((Preferences*)data)->sidebar_icon_pressed(w); }

static gint delete_event(GtkWidget *widget, GdkEvent *event, gpointer data)
{ ((Preferences*)data)->hide(); return (TRUE); }

Preferences::Preferences(Main *m)
{
	GladeXML *gxml;

	main = m;
	state = 0;

	gchar *fname = user_file("preferences.glade");
	gxml = glade_xml_new (fname, "preferences");
	if (!gxml)
	{
		g_warning("Failed to load file graph.glade");
		return;
	}
	window = glade_xml_get_widget (gxml, "preferences");
	button1 = glade_xml_get_widget (gxml, "button1");
	button2 = glade_xml_get_widget (gxml, "button2");
	sidebar = glade_xml_get_widget (gxml, "sidebar");
	notebook = glade_xml_get_widget (gxml, "notebook");
	entry9 = glade_xml_get_widget (gxml, "entry9");
	entry10 = glade_xml_get_widget (gxml, "entry10");
	entry11 = glade_xml_get_widget (gxml, "entry11");
	entry13 = glade_xml_get_widget (gxml, "entry13");
	clist2 = glade_xml_get_widget (gxml, "clist2");
	clist3 = glade_xml_get_widget (gxml, "clist3");
	clist4 = glade_xml_get_widget (gxml, "clist4");
	entry5 = glade_xml_get_widget (gxml, "entry5");
	button7 = glade_xml_get_widget (gxml, "button7");
	button8 = glade_xml_get_widget (gxml, "button8");
	gtk_object_unref (GTK_OBJECT (gxml));
	g_free(fname);

  gtk_widget_realize (window);
	window_icon(window, licon_xpm);
	gdk_window_set_icon_name(window->window, "Preferences");
	gtk_signal_connect (GTK_OBJECT (window), "delete_event",
		GTK_SIGNAL_FUNC(delete_event), this);

	build_sidebar();

	gtk_signal_connect(GTK_OBJECT(clist4), "select_row",
		GTK_SIGNAL_FUNC(select_community_cb), this);
	gtk_signal_connect(GTK_OBJECT(clist4), "unselect_row",
		GTK_SIGNAL_FUNC(unselect_community_cb), this);
  gtk_signal_connect (GTK_OBJECT (entry5), "activate",
		GTK_SIGNAL_FUNC (add_community_cb), this);
  gtk_signal_connect (GTK_OBJECT (button8), "clicked",
		GTK_SIGNAL_FUNC (add_community_cb), this);
  gtk_signal_connect (GTK_OBJECT (button7), "clicked",
		GTK_SIGNAL_FUNC (del_community_cb), this);
  gtk_signal_connect(GTK_OBJECT(button1), "clicked",
		GTK_SIGNAL_FUNC(accept_cb), this);
  gtk_signal_connect(GTK_OBJECT(button2), "clicked",
		GTK_SIGNAL_FUNC(cancel_cb), this);

	load();		// load user preferences
	save();		// save to apply any new preferences

	unselect_community();
}

Preferences::~Preferences()
{
}

void Preferences::show()
{
	// only load if actually opening the window

	if (state==0) load();
	state = 1;
	gtk_widget_show(window);
	gdk_window_raise(window->window);
}

void Preferences::hide()
{
	state = 0;
	gtk_widget_hide(window);
}

//////////////////////////////////////////////////////////////////////
// sidebar
//////////////////////////////////////////////////////////////////////

void Preferences::build_sidebar()
{
	GtkWidget *panel;

	panel = sidebar_button("Application");
	sidebar_icon(panel, "Session", gnome_session_xpm, 4);
	sidebar_icon(panel, "Import / Export", faq_48_xpm, 6);
	sidebar_icon(panel, "Helper", faq_48_xpm, 5);
	sidebar_pad(panel);
	panel = sidebar_button("Graphs");
	sidebar_icon(panel, "Variables", gnome_stripchart_xpm, 1);
	sidebar_icon(panel, "Sample Rate", gnome_clock_xpm, 2);
	sidebar_pad(panel);
	panel = sidebar_button("Network");
	sidebar_icon(panel, "Community", helix_setup_users3_xpm, 3);
	sidebar_pad(panel);

	current_panel = NULL;
}

GtkWidget *Preferences::sidebar_button(char *label)
{
	GtkWidget *button, *button2, *panel;

	button = gtk_button_new_with_label (label);
	gtk_widget_show (button);
	panel = gtk_vbox_new (FALSE, 0);
	gtk_box_pack_start(GTK_BOX(sidebar), button, FALSE, TRUE, 0);
	gtk_box_pack_start(GTK_BOX(sidebar), panel, TRUE, TRUE, 2);
	gtk_object_set_data (GTK_OBJECT (button), "panel", panel);
	gtk_signal_connect (GTK_OBJECT (button), "clicked",
		GTK_SIGNAL_FUNC(sidebar_button_pressed_cb), this);

	// icon buttons are in a radio group, we need to create a hidden
	// button to make it look like there are no buttons selected at start
	button_group = NULL;
	button2 = gtk_radio_button_new(button_group);
	button_group = gtk_radio_button_group(GTK_RADIO_BUTTON (button2));
	gtk_box_pack_start (GTK_BOX (panel), button2, FALSE, TRUE, 0);
	gtk_button_clicked (GTK_BUTTON (button2));

	return panel;
}

void Preferences::sidebar_icon(GtkWidget *w, char *label, char **icon, int pg)
{
	GtkWidget *button, *vbox, *pixmap, *widget;

	button = gtk_radio_button_new (button_group);
	gtk_button_set_relief (GTK_BUTTON (button), GTK_RELIEF_NONE);
	gtk_toggle_button_set_mode (GTK_TOGGLE_BUTTON (button), FALSE);
	button_group = gtk_radio_button_group(GTK_RADIO_BUTTON (button));
	gtk_object_set_data (GTK_OBJECT (button), "page", GINT_TO_POINTER(pg));
	gtk_signal_connect (GTK_OBJECT (button), "clicked",
		GTK_SIGNAL_FUNC(sidebar_icon_pressed_cb), this);
	gtk_box_pack_start (GTK_BOX (w), button, FALSE, TRUE, 0);
	gtk_widget_show (button);

	vbox = gtk_vbox_new (FALSE, 0);
	gtk_container_add (GTK_CONTAINER (button), vbox);
	gtk_widget_show (vbox);

	pixmap = create_pixmap(window, icon);
	gtk_box_pack_start (GTK_BOX (vbox), pixmap, FALSE, FALSE, 0);
	gtk_widget_show (pixmap);

	widget = gtk_label_new (label);
	gtk_misc_set_alignment (GTK_MISC (widget), 0.5, 0.5);
	gtk_label_set_line_wrap (GTK_LABEL (widget), FALSE);
	gtk_box_pack_start (GTK_BOX (vbox), widget, FALSE, TRUE, 0);
	gtk_widget_show (widget);
}

void Preferences::sidebar_pad(GtkWidget *w)
{
	GtkWidget *widget;

	widget = gtk_invisible_new();
	gtk_box_pack_end(GTK_BOX (w), widget, TRUE, TRUE, 0);
	gtk_widget_show(widget);
	gtk_object_set_data (GTK_OBJECT (w), "group", button_group);
}

void Preferences::sidebar_button_pressed(GtkWidget *w)
{
	GSList *grp;
	GtkWidget *panel;
	int page = 0;

	panel = GTK_WIDGET(gtk_object_get_data(GTK_OBJECT (w), "panel"));
	if (current_panel != NULL)
	{
		gtk_widget_hide (current_panel);
	}
	if (current_panel != panel)
	{
		current_panel = panel;
		gtk_widget_show (panel);
		// set notebook page based on radio button currently set
		grp = (GSList*)(gtk_object_get_data(GTK_OBJECT (panel), "group"));
		while (grp!=NULL)
		{
			if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON (grp->data)))
			{
				page = GPOINTER_TO_INT(gtk_object_get_data(GTK_OBJECT(grp->data), "page"));
			}
			grp = g_slist_next(grp);
		}
	}else
	{
		current_panel = NULL;
	}
	gtk_notebook_set_page(GTK_NOTEBOOK(notebook), page);
}

void Preferences::sidebar_icon_pressed(GtkWidget *w)
{
	int page = GPOINTER_TO_INT(gtk_object_get_data(GTK_OBJECT(w), "page"));
	gtk_notebook_set_page(GTK_NOTEBOOK(notebook), page);
}

/////////////////////////////////////////////////////////////////

void Preferences::select_community(int r)
{
	community_row = r;
	gtk_widget_set_sensitive(button7, TRUE);

}

void Preferences::unselect_community()
{
	gtk_widget_set_sensitive(button7, FALSE);
	community_row = -1;
}

void Preferences::add_community()
{
	gchar *data[1];

	gtk_clist_freeze(GTK_CLIST(clist4));
	data[0] = strdup(gtk_entry_get_text(GTK_ENTRY(entry5)));
	gtk_clist_append(GTK_CLIST(clist4), data);
	delete data[0];
	gtk_clist_set_sort_column (GTK_CLIST(clist4), 0);
	gtk_clist_sort (GTK_CLIST (clist4));
	gtk_clist_thaw(GTK_CLIST(clist4));
	gtk_entry_set_text(GTK_ENTRY(entry5), "");
}

void Preferences::del_community()
{
	gtk_clist_remove(GTK_CLIST(clist4), community_row);
}

/////////////////////////////////////////////////////////////////
// data
/////////////////////////////////////////////////////////////////

void Preferences::load()
{
	set_editable_from_config(entry9,
		"/nomad/helper/ping=xterm -e ping %s");
	set_editable_from_config(entry10,
		"/nomad/helper/telnet=xterm -e telnet %s");
	set_editable_from_config(entry11,
		"/nomad/helper/traceroute=xterm -e nomad-traceroute %s");
	set_editable_from_config(entry13,
		"/nomad/session/startup=");
	set_community_from_config();
	set_graph_from_config();
}

void Preferences::save()
{
	set_config_from_editable(entry9, "/nomad/helper/ping");
	set_config_from_editable(entry10, "/nomad/helper/telnet");
	set_config_from_editable(entry11, "/nomad/helper/traceroute");
	set_config_from_editable(entry13, "/nomad/session/startup");
	set_config_from_community();
	gnome_config_sync();
}

void Preferences::load_settings()
{
}

void Preferences::save_settings()
{
/*
	gchar *str;

	gnome_config_set_string("/nomad/settings/sidepanel", main->gui->monitor->side_panel);
	g_free(str);
*/
}

void Preferences::set_config_from_community()
{
	gint i, num;
	gchar *data;
	gchar key[64];

	gnome_config_clean_section("/nomad/community");
	num = GTK_CLIST(clist4)->rows;
	for (i=0; i<num; i++)
	{
		gtk_clist_get_text(GTK_CLIST(clist4), i, 0, &data);
		g_snprintf(key, 63, "/nomad/community/%d", i);
		gnome_config_set_string(key, data);
	}
}

void Preferences::set_community_from_config()
{
	gpointer itr;
	gchar *data[1];
	gchar *key, *val;
	gint flag = 0;

	gtk_clist_freeze(GTK_CLIST(clist4));
	gtk_clist_clear(GTK_CLIST(clist4));

	itr = gnome_config_init_iterator("/nomad/community");
	while (itr!=NULL)
	{
		itr = gnome_config_iterator_next(itr, &key, &val);
		if (val!=NULL)
		{
			data[0] = strdup(val);
			gtk_clist_append(GTK_CLIST(clist4), data);
			delete data[0];
			flag = 1;
		}
		g_free(key);
		g_free(val);
	}

	if (flag == 0)
	{
		data[0] = strdup("public");	// default
		gtk_clist_append(GTK_CLIST(clist4), data);
		delete data[0];
	}

	gtk_clist_set_sort_column (GTK_CLIST (clist4), 0);
	gtk_clist_sort (GTK_CLIST (clist4));
	gtk_clist_thaw(GTK_CLIST(clist4));
}

void Preferences::set_graph_from_config()
{
	gchar *data[4], *datab[2];
	char buf[16];
	GraphIO gio;
	int i;

	gio.read_global();

	gtk_clist_freeze(GTK_CLIST(clist2));
	gtk_clist_clear(GTK_CLIST(clist2));
	for (i=0; i<gio.num_global_values; i++)
	{
		data[0] = strdup(gio.global_types[i].oid);
		data[1] = strdup(gio.global_types[i].label);
		data[2] = strdup("");
		data[3] = strdup("");
		gtk_clist_append(GTK_CLIST(clist2), data);
		delete data[0]; delete data[1];
		delete data[2]; delete data[3];
	}
	gtk_clist_thaw(GTK_CLIST(clist2));

	gtk_clist_freeze(GTK_CLIST(clist3));
	gtk_clist_clear(GTK_CLIST(clist3));
	for (i=0; i<gio.num_global_blocks; i++)
	{
		sprintf(buf, "%lu", gio.global_blocks[i].gap);
		datab[0] = strdup(buf);
		sprintf(buf, "%lu", gio.global_blocks[i].howlong);
		datab[1] = strdup(buf);
		datab[2] = strdup("");
		datab[3] = strdup("");
		gtk_clist_append(GTK_CLIST(clist3), datab);
		delete datab[0]; delete datab[1];
		delete datab[2]; delete datab[3];
	}
	gtk_clist_thaw(GTK_CLIST(clist3));
}

gchar *get_community_list_string()
{
	gpointer itr;
	gchar *key, *val;
	gint flag = 0;
	gchar *str;

	itr = gnome_config_init_iterator("/nomad/community");
	while (itr!=NULL)
	{
		itr = gnome_config_iterator_next(itr, &key, &val);
		g_free(key);
		if (val!=NULL)
		{
			if (flag==0)
				str = g_strdup(val);
			else
			{
				key = g_strjoin(" ", str, val, NULL);
				g_free(str);
				str = key;
			}
			flag = 1;
		}
		g_free(val);
	}

	if (flag == 0)
	{
		str = g_strdup("public");
	}
	return str;
}

// Save & Load Routines

void set_editable_from_config(GtkWidget *w, gchar *config)
{
	gchar *str;

	str = gnome_config_get_string(config);
	gtk_entry_set_text(GTK_ENTRY(w), str);
	g_free(str);
}

void set_config_from_editable(GtkWidget *w, gchar *config)
{
	gchar *str;

	str = gtk_editable_get_chars(GTK_EDITABLE(w), 0, -1);
	gnome_config_set_string(config, str);
	g_free(str);
}

// Searches a number of possible locations for the file
// returns full path to file or NULL if not found

gchar *user_file(gchar *fname)
{
	gchar *name[7];
	gchar *file = NULL;
	int i;

	name[0] = g_strconcat(g_get_home_dir(), "/", DATA_DIR, "/", fname, NULL);
	name[1] = g_strdup(fname);
	name[2] = g_strconcat("../", fname, NULL);
	name[3] = g_strconcat("../", fname, NULL);
	name[4] = g_strconcat ("ui/", fname, NULL);
	name[5] = g_strconcat ("../ui/", fname, NULL);
	name[6] = g_strconcat (SHARE_DIR, "/", fname, NULL);

	for (i=0; i<7; i++)
	{
		if (g_file_exists(name[i])) { file = g_strdup(name[i]); i=7; }
	}

	for (i=0; i<7; i++)
	{
		g_free(name[i]);
	}

	return file;
}

