/* session.cc
 * Copyright (C) 2001 The University of Newcastle upon Tyne
 *
 * Written by Paul Coates (Paul.Coates@ncl.ac.uk)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 */

#include <gtk/gtk.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <pwd.h>
#include "filter.h"
#include "filteritem.h"
#include "session.h"
#include "sessionlist.h"
#include "scheduler.h"
#include "data.h"
#include "main.h"
#include "gui.h"
#include "monitor.h"
#include "diagram.h"
#include "view.h"
#include "header.h"

static void close_cb(GtkWidget *w, gpointer data)
{	gtk_widget_hide(GTK_WIDGET(data)); }
static void action_cb(GtkWidget *w, gpointer data)
{ ((Session*)data)->session_action(); }
static void have_title_cb(GtkWidget *w, gpointer data)
{
	((Session*)data)->save_msg.hide();
	((Session*)data)->have_title();
}
static void unique_name_cb(GtkWidget *w, gpointer data)
{
	((Session*)data)->save_session_list.hide();
	((Session*)data)->unique_name();
}
static void save_session_cb(GtkWidget *w, gpointer data)
{
	Session* s = (Session*) data;
	strcpy(s->session_name, s->save_session_list.getName());
	s->save_session();
}
static void load_session_cb(GtkWidget *w, gpointer data)
{
	Session* s = (Session*) data;
	s->load_session(s->load_session_list.getName());
}

Session::Session(Main *m)
{
	struct stat statbuf;
	struct passwd *pwd;
	char buf[256];
	char *md;

	md = getenv("NOMAD_DIR");
	if (md!=NULL) { strcpy(datadir, md); } else
	{ pwd = getpwuid(getuid());
		sprintf(datadir, "%s/%s", pwd->pw_dir, DATA_DIR);
	}

	sprintf(buf, "%s", datadir);
	if (stat(buf, &statbuf) == -1) mkdir(buf, 488); // rwxr x
	sprintf(buf, "%s/sessions", datadir);
	if (stat(buf, &statbuf) == -1) mkdir(buf, 488); // rwxr x

	main = m;
	filters.empty();
	views.empty();
	data = new Data();

	action = 0;
	dirty = 'n';
	strcpy(session_name, "");

	save_msg.setTitle("Save Session");
	save_msg.setLabel("Save current session?");
	save_msg.setButtons(1);
	gtk_signal_connect (GTK_OBJECT (save_msg.button_yes), "clicked",
		GTK_SIGNAL_FUNC (have_title_cb), this);
	gtk_signal_connect (GTK_OBJECT (save_msg.button_no), "clicked",
		GTK_SIGNAL_FUNC (action_cb), this);
	gtk_signal_connect (GTK_OBJECT (save_msg.button_cancel), "clicked",
		GTK_SIGNAL_FUNC (close_cb), save_msg.window);

	overwrite_msg.setTitle("Overwrite Session");
	overwrite_msg.setLabel("Session already exists.\nOverwrite session?");
	overwrite_msg.setButtons(2);
	gtk_signal_connect (GTK_OBJECT (overwrite_msg.button_yes), "clicked",
		GTK_SIGNAL_FUNC (save_session_cb), this);
	gtk_signal_connect (GTK_OBJECT (overwrite_msg.button_no), "clicked",
		GTK_SIGNAL_FUNC (close_cb), overwrite_msg.window);

	failure_msg.setTitle("Load Failed");
	failure_msg.setButtons(3);
	gtk_signal_connect (GTK_OBJECT (failure_msg.button_yes), "clicked",
		GTK_SIGNAL_FUNC (close_cb), failure_msg.window);

	load_session_list.setTitle("Load Session");
	gtk_signal_connect (GTK_OBJECT (load_session_list.entry), "activate",
		GTK_SIGNAL_FUNC(load_session_cb), this);
	gtk_signal_connect (GTK_OBJECT (load_session_list.button_accept), "clicked",
		GTK_SIGNAL_FUNC (load_session_cb), this);
	gtk_signal_connect (GTK_OBJECT (load_session_list.button_cancel), "clicked",
		GTK_SIGNAL_FUNC (close_cb), load_session_list.window);

	save_session_list.setTitle("Save Session");
	gtk_signal_connect (GTK_OBJECT (save_session_list.entry), "activate",
		GTK_SIGNAL_FUNC(unique_name_cb), this);
	gtk_signal_connect (GTK_OBJECT (save_session_list.button_accept), "clicked",
		GTK_SIGNAL_FUNC (unique_name_cb), this);
	gtk_signal_connect (GTK_OBJECT (save_session_list.button_cancel), "clicked",
		GTK_SIGNAL_FUNC (close_cb), save_session_list.window);
}

Session::~Session()
{
	if (data!=NULL) delete data;
}

void Session::new_session(int type)
{
	action = type;
//	printf("Is dirty flag set?\n");
	if (dirty == 'y')
	{
//		if ((options->auto_save_on_quit) && (type==3))
//		{
//			printf("Doing autosaveonquit\n");
//			have_title();
//		}else
//			save_msg.show();
	}else
		session_action();
}

void Session::have_title()
{
	save_msg.hide();
//	printf("Does the current session have a title?\n");
	if (strcmp(session_name, "")==0)
	{
		save_session_list.init(datadir);
		save_session_list.show();
	}else
		save_session();
}

void Session::session_action()
{
	Link *ln;
	Filter *f;
	View *v;

//	main->setTitle();

	save_msg.hide();
	if (action==1)	// new
	{
		for (ln=filters.get_head(); ln!=NULL; ln=ln->next)
		{ f = (Filter*) ln->item; if (f!=NULL) delete f; } filters.empty();
		for (ln=views.get_head(); ln!=NULL; ln=ln->next)
		{ v = (View*) ln->item; if (v!=NULL) delete v; } views.empty();
		if (data!=NULL) delete data;
		data = new Data();
		dirty = 'n';
		strcpy(session_name, "");
		main->gui->update_views();
		main->gui->update_lists();
	}else
	if (action==2)	// open
	{
		load_session_list.init(datadir);
		load_session_list.show();
//		printf("Session list to load etc.\n");
	}
	main->setTitle();
}

void Session::unique_name()
{
	char buf[256];
	SessionName *nm;
	Link *ln;
	int match;

//	printf("Does it have a unique name?\n");

	// extract name
	strcpy(buf, save_session_list.getName());
	if (strcmp(buf, "")==0) return;

	match = 0;
	for (ln=save_session_list.names.get_head(); ln!=NULL; ln=ln->next)
	{
		nm = (SessionName*) ln->item;
		if (strcmp(buf, nm->label)==0) match=1;
	}
	if ((match==1) && (strcmp(buf, session_name)!=0))
	{
		overwrite_msg.show();
	}else
	{
		strcpy(session_name, save_session_list.getName());
		save_session();
	}
}

void Session::save_session()
{
	struct stat statbuf;
	Filter *f;
	View *v;
	Link *ln;
	xmlDocPtr doc;
	char fname[256];

	// if fails during save call failed dialog
//	printf("Doing save\n");
	overwrite_msg.hide();

	// make sure directory exists
	sprintf(fname, "%s/sessions/%s", datadir, session_name);
	if (stat(fname, &statbuf) == -1) mkdir(fname, 488); // rwxr x

	// save topology
	sprintf(fname, "%s/sessions/%s/network", datadir, session_name);
	data->savexml(fname);

	// save filters
	doc = xmlNewDoc((xmlChar*)"1.0");
	doc->root = xmlNewDocNode(doc, NULL, (xmlChar*)"Nomad_Filters", NULL);
	for (ln=filters.get_head(); ln!=NULL; ln=ln->next)
	{
		f = (Filter*) ln->item;
		f->savexml(doc->root);
	}
	sprintf(fname, "%s/sessions/%s/filters", datadir, session_name);
	xmlSaveFile(fname, doc);
	xmlFreeDoc(doc);

	// save views
	doc = xmlNewDoc((xmlChar*)"1.0");
	doc->root = xmlNewDocNode(doc, NULL, (xmlChar*)"Nomad_Views", NULL);
	for (ln=views.get_head(); ln!=NULL; ln=ln->next)
	{
		v = (View*) ln->item;
		v->savexml(doc->root);
	}
	sprintf(fname, "%s/sessions/%s/views", datadir, session_name);
	xmlSaveFile(fname, doc);
	xmlFreeDoc(doc);

	dirty = 'n';
	session_action();
}

void Session::load_session(char *name)
{
	xmlDocPtr doc, doc2;
	xmlNodePtr xmlnode;
	char buf[256];
	Filter *f;
	View *v;
	Link *ln;
	Data *d;
	int forceflag;
	float t1, t2, sc;

	load_session_list.hide();

	if (strcmp(name, "")==0)
	{
		sprintf(buf, "Empty session name");
		failure_msg.setLabel(buf);
		failure_msg.show();
		return;
	}

	sprintf(buf, "%s/sessions/%s/filters", datadir, name);
	doc = xmlParseFile(buf);

	if (!doc || !doc->root || !doc->root->name ||
			strcmp((char*)doc->root->name, "Nomad_Filters")!=0)
	{
		xmlFreeDoc(doc);
		sprintf(buf, "Failed to load filters for session \"%s\"", name);
		failure_msg.setLabel(buf);
		failure_msg.show();
	}

	sprintf(buf, "%s/sessions/%s/views", datadir, name);
	doc2 = xmlParseFile(buf);

	if (!doc2 || !doc2->root || !doc2->root->name ||
			strcmp((char*)doc2->root->name, "Nomad_Views")!=0)
	{
		xmlFreeDoc(doc2);
		sprintf(buf, "Failed to load views for session \"%s\"", name);
		failure_msg.setLabel(buf);
		failure_msg.show();
	}

	d = new Data();
	sprintf(buf, "%s/sessions/%s/network", datadir, name);
	d->loadxml(buf);
	// if we are just updating then copy over visual stuff
	// if we are loading a new diagram leave random OR
	//   if has a default view apply view

	// if everything ok so far delete existing structures
	for (ln=filters.get_head(); ln!=NULL; ln=ln->next)
	{ f = (Filter*) ln->item; if (f!=NULL) delete f; }
	filters.empty();
	for (ln=views.get_head(); ln!=NULL; ln=ln->next)
	{ v = (View*) ln->item; if (v!=NULL) delete v; }
	views.empty();
	delete data;

	// use new data

	data = d;	

	if (doc && doc->root && doc->root->name &&
			strcmp((char*)doc->root->name, "Nomad_Filters")==0)
	{
		xmlnode = doc->root->childs;
		while (xmlnode != NULL)
		{
			if (xmlnode->name!=NULL)
			{
				if (strcmp((char*)xmlnode->name, "Filter")==0)
				{
					f = new Filter();
					f->loadxml(xmlnode);
					filters.tag(f);
					while(gtk_events_pending()) gtk_main_iteration();
				}
			}
			xmlnode = xmlnode->next;
		}
		xmlFreeDoc(doc);
	}

	if (doc2 && doc2->root && doc2->root->name &&
			strcmp((char*)doc2->root->name, "Nomad_Views")==0)
	{
		xmlnode = doc2->root->childs;
		while (xmlnode != NULL)
		{
			if (xmlnode->name!=NULL)
			{
				if (strcmp((char*)xmlnode->name, "View")==0)
				{
					v = new View();
					v->loadxml(xmlnode);
					views.tag(v);
					while(gtk_events_pending()) gtk_main_iteration();
				}
			}
			xmlnode = xmlnode->next;
		}
		xmlFreeDoc(doc2);
	}

	// apply default view (only if this is a new session and not an update)
	for (ln=views.get_head(); ln!=NULL; ln=ln->next)
	{
		v = (View*) ln->item;
		if (strcmp(v->name, data->default_view)==0)
		{
			v->copy_to_topology(data);
		}
	}

	forceflag = main->gui->monitor->viewer->animate_version;
	main->gui->monitor->viewer->animate_version = 0;

	data->sizeshow();
	t1 = float((data->width)/(float)(main->gui->monitor->viewer->width-(BORDER*2)));
	t2 = float((data->height)/(float)(main->gui->monitor->viewer->height-(BORDER*2)));
	sc = (t1>t2)?t1:t2;
	if (sc<1.0)
		main->gui->monitor->viewer->scale = 1.0;
	else
		main->gui->monitor->viewer->scale = (float((int(sc*100))+1))/100;
	main->gui->monitor->viewer->xpos = (long) data->minx+(data->width/2);
	main->gui->monitor->viewer->ypos = (long) data->miny+(data->height/2);
	GTK_ADJUSTMENT(main->gui->monitor->viewer->hscale3_adj)->value = (float((int(sc*100))+1))/100;;
	main->gui->monitor->viewer->setScale(main->gui->monitor->viewer->scale);

	main->gui->monitor->viewer->animate_version = forceflag;

	strcpy(session_name, name);
	main->setTitle();
	main->gui->update_views();
	main->gui->update_lists();

	// ping stuff
	save_device_file();																// write out a devices file
	main->scheduler->addTask(session_name, 2, 0, 0);	// then assign a ping task
}

void Session::save_device_file()
{
	struct stat statbuf;
	FILE *f;
	char buf[256];
	Link *ln;
	Device *d;

	sprintf(buf, "%s/sessions/%s", datadir, session_name);
	if (stat(buf, &statbuf) == -1) mkdir(buf, 488); // rwxr x
	sprintf(buf, "%s/sessions/%s/devices", datadir, session_name);
	f = fopen(buf, "w");
	if (f)
	{
		for (ln=data->devices.get_head(); ln!=NULL; ln=ln->next)
		{
			d = (Device*) ln->item;
			if (d->type!='u')
			{
				fprintf(f, "%c,%s,%s\n",
					d->grab_stats, d->pingname.getstring(), d->community);
			}
		}
		fclose(f);
	}
}

void Session::quit_session()
{
	// so we can save session before we quit
	new_session(3);
}

int Session::uniquefiltername(Filter *orig, char *name)
{
	Link *ln;
	Filter *f;

	for (ln=filters.get_head(); ln!=NULL; ln=ln->next)
	{
		f = (Filter*) ln->item;
		if ((orig!=f) && (strcmp(name, f->name)==0))
			return 0;
	}
	return 1;
}

int Session::uniqueviewname(char *name)
{
	Link *ln;
	View *v;

	for (ln=views.get_head(); ln!=NULL; ln=ln->next)
	{
		v = (View*) ln->item;
		if (strcmp(name, v->name)==0)
			return 0;
	}
	return 1;
}

