/* stats.cc
 * Copyright (C) 2001 The University of Newcastle upon Tyne
 *
 * Written by Paul Coates (Paul.Coates@ncl.ac.uk)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 */

#include <glade/glade-xml.h>
#include <gtk/gtk.h>
#include <sys/time.h>
#include <sys/stat.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <stdio.h>
#include <math.h>
#include <pwd.h>
#include "main.h"
#include "session.h"
#include "data.h"
#include "gui.h"
#include "device.h"
#include "interface.h"
#include "licon.xpm"
#include "intup.xpm"
#include "intdown.xpm"
#include "stats.h"
#include "dataset.h"
#include "graphio.h"
#include "preferences.h"

static void on_Close_activate(GtkWidget *w, gpointer data)
{ ((Stats*)data)->hide(); }
static void start_left_cb(GtkWidget *w, gpointer data)
{ ((Stats*)data)->start_left(); }
static void start_right_cb(GtkWidget *w, gpointer data)
{ ((Stats*)data)->start_right(); }
static void middle_left_cb(GtkWidget *w, gpointer data)
{ ((Stats*)data)->middle_left(); }
static void middle_right_cb(GtkWidget *w, gpointer data)
{ ((Stats*)data)->middle_right(); }
static void end_left_cb(GtkWidget *w, gpointer data)
{ ((Stats*)data)->end_left(); }
static void end_right_cb(GtkWidget *w, gpointer data)
{ ((Stats*)data)->end_right(); }
static void device_list_cb(GtkWidget *w, gpointer data)
{ ((Stats*)data)->device_list(); }
//static void interface_list_cb(GtkWidget *w, gpointer data)
//{ ((Stats*)data)->interface_list(); }
static void interface_clist_cb(GtkWidget *clist, gint row, gint column, GdkEventButton *event, gpointer data)
{
	Stats *m = (Stats*) data;
	Interface *in;

	in = (Interface*) gtk_clist_get_row_data(GTK_CLIST(clist), row);
//	m->draw();
	m->current_interface = in;
	m->load();
}

static void interface_clist_cb2(GtkWidget *clist, gint row, gint column, GdkEventButton *event, gpointer data)
{
	Stats *m = (Stats*) data;
	m->current_interface = NULL;
	m->draw();
}




static void radio_cb(GtkWidget *w, gpointer data)
{
	Stats *s = (Stats*) data;
	s->scale = (w == s->radio1)?0:1;
	s->draw();
}

static gint delete_event(GtkWidget *widget, GdkEvent *event, gpointer data)
{
	((Stats*)data)->hide();
	return (TRUE);
}

static gint expose_event(GtkWidget *widget, GdkEventExpose *event, gpointer data)
{
	Stats *m = (Stats*)data;
	if (m->canvas!=NULL)
	{
		gdk_draw_pixmap(widget->window,
			widget->style->fg_gc[GTK_WIDGET_STATE (widget)],
			m->canvas, event->area.x, event->area.y, event->area.x, event->area.y,
			event->area.width, event->area.height);
	}
	return FALSE;
}

static gint configure_event(GtkWidget *widget, GdkEventConfigure *event, gpointer data)
{ ((Stats*)data)->draw(); return TRUE; }

int compare_router_names(Link *a, Link *b)
{
	Device *r1, *r2;

	r1 = (Device*) a->item;
	r2 = (Device*) b->item;

	return (strcmp(r1->name, r2->name));
}

Stats::Stats(Main *m)
{
	GList *glist = NULL;
	gchar *datacol[2];
	Link *ln;
	Device *d;
	GladeXML *gxml;
	GdkColor col;
	int i;

	main = m;

	canvas = NULL;
	height = width = 0;
	error = 0;
	current_interface = NULL;
	current_router = NULL;
	day_range = 0;
	scale = 0;
	data = new DataSet();

//	iface = NULL;
//	strcpy(current_file, "");

	gchar *fname = user_file("graph.glade");
	gxml = glade_xml_new (fname, "graph");
	if (!gxml)
	{
		g_warning("Failed to load file graph.glade");
		return;
	}

	window = glade_xml_get_widget (gxml, "graph");
	clist5 = glade_xml_get_widget (gxml, "clist5");
	clist6 = glade_xml_get_widget (gxml, "clist6");
	combo3 = glade_xml_get_widget (gxml, "combo3");
	button1 = glade_xml_get_widget (gxml, "button1");
	button2 = glade_xml_get_widget (gxml, "button2");
	button3 = glade_xml_get_widget (gxml, "button3");
	button4 = glade_xml_get_widget (gxml, "button4");
	button5 = glade_xml_get_widget (gxml, "button5");
	button6 = glade_xml_get_widget (gxml, "button6");
	button13 = glade_xml_get_widget (gxml, "button13");
	radio1 = glade_xml_get_widget (gxml, "radio1");
	radio2 = glade_xml_get_widget (gxml, "radio2");
	drawingarea1 = glade_xml_get_widget (gxml, "drawingarea1");
	gtk_object_unref (GTK_OBJECT (gxml));
	g_free(fname);

	gtk_widget_set_usize (window, 480, 320);
	gtk_widget_realize (window);
	window_icon(window, licon_xpm);
	gdk_window_set_icon_name(window->window, "Preferences");
	gtk_signal_connect (GTK_OBJECT (window), "delete_event",
		GTK_SIGNAL_FUNC(delete_event), this);
	gtk_widget_realize(drawingarea1);

  gtk_signal_connect(GTK_OBJECT(GTK_COMBO(combo3)->list), "selection_changed",
		GTK_SIGNAL_FUNC (device_list_cb), this);

	gtk_signal_connect (GTK_OBJECT (button1), "pressed",
		GTK_SIGNAL_FUNC (start_left_cb), this);
	gtk_signal_connect (GTK_OBJECT (button2), "pressed",
		GTK_SIGNAL_FUNC (start_right_cb), this);
	gtk_signal_connect (GTK_OBJECT (button3), "pressed",
		GTK_SIGNAL_FUNC (middle_left_cb), this);
	gtk_signal_connect (GTK_OBJECT (button4), "pressed",
		GTK_SIGNAL_FUNC (middle_right_cb), this);
	gtk_signal_connect (GTK_OBJECT (button5), "pressed",
		GTK_SIGNAL_FUNC (end_left_cb), this);
	gtk_signal_connect (GTK_OBJECT (button6), "pressed",
		GTK_SIGNAL_FUNC (end_right_cb), this);
	gtk_signal_connect (GTK_OBJECT (radio1), "pressed",
		GTK_SIGNAL_FUNC (radio_cb), this);
	gtk_signal_connect (GTK_OBJECT (radio2), "pressed",
		GTK_SIGNAL_FUNC (radio_cb), this);

	gtk_signal_connect(GTK_OBJECT(clist6), "select_row",
		GTK_SIGNAL_FUNC(interface_clist_cb), this);
	gtk_signal_connect(GTK_OBJECT(clist6), "unselect_row",
		GTK_SIGNAL_FUNC(interface_clist_cb2), this);

	gtk_signal_connect (GTK_OBJECT (button13), "pressed",
		GTK_SIGNAL_FUNC (on_Close_activate), this);
	gtk_signal_connect (GTK_OBJECT (drawingarea1), "expose_event",
		(GtkSignalFunc) expose_event, this);
	gtk_signal_connect (GTK_OBJECT(drawingarea1),"configure_event",
		(GtkSignalFunc) configure_event, this);

	alloc_color(lightgray, 49151, 49151, 49151);
	alloc_color(gray, 32767, 32767, 32767);
	alloc_color(darkgray, 16383, 16383, 16383);
	alloc_color(black, 0, 0, 0);

	gc      = gdk_gc_new(window->window);
	gridgc  = gdk_gc_new(window->window);
	gdk_gc_set_line_attributes(gc, 1, GDK_LINE_SOLID, GDK_CAP_BUTT,
		GDK_JOIN_MITER);

	main->session->data->devices.sort(compare_router_names, BUBBLE_SORT);

	for (ln=main->session->data->devices.get_head(); ln!=NULL; ln=ln->next)
	{
		d = (Device*) ln->item;
		glist = g_list_append(glist, d->name);
	}
	gtk_combo_set_popdown_strings(GTK_COMBO(combo3), glist);
	g_list_free(glist);

	// load key from global file
	if (gio.read_global())
	{
		gtk_clist_freeze(GTK_CLIST(clist5));
		gtk_clist_clear(GTK_CLIST(clist5));

		for (i=0; i<gio.num_global_values; i++)
		{
			datacol[0] = strdup("");
			datacol[1] = strdup(gio.global_types[i].label);
			gtk_clist_append(GTK_CLIST(clist5), datacol);
			alloc_color(col, gio.global_types[i].r,
				gio.global_types[i].g, gio.global_types[i].b);
			gtk_clist_set_foreground(GTK_CLIST(clist5), i, &col);
			delete datacol[0]; delete datacol[1];
		}
		gtk_clist_thaw(GTK_CLIST(clist5));
	}
}

Stats::~Stats()
{
	if (gc) gdk_gc_unref(gc);
	if (data != NULL) delete data;
	if (gridgc) gdk_gc_unref(gridgc);
//	if (iface != NULL)
//		delete [] iface;
}

void Stats::show()
{
	gtk_widget_show(window);
	gdk_window_raise(window->window);
	load();
}

void Stats::hide()
{
	gtk_widget_hide(window);
}

void Stats::device_list()
{
	Device *d;
	Link *ln;

	char *string = gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(combo3)->entry));

	for (ln=main->session->data->devices.get_head(); ln!=NULL; ln=ln->next)
	{
		d = (Device*) ln->item;
		if (strcmp(d->name, string)==0)
		{
			current_router = d;
			current_interface = NULL;
			set_interface_list();
		}
	}
	load();
}

void Stats::set_interface_list()
{
//	GList *combo_items = NULL;
	gchar *data[2];
	Link *ln;
	Interface *in;
	int row = 0;

	if (current_router == NULL) return;
	if (current_router->interfaces.size()<1) return;

/*
	combo_items = g_list_append(combo_items, (void*)"All Interfaces");
	for (ln=current_router->interfaces.get_head(); ln!=NULL; ln=ln->next)
	{
		in = (Interface*) ln->item;
		if (strcmp(in->descr, "")!=0)
			combo_items = g_list_append(combo_items, in->descr);
	}
	gtk_combo_set_popdown_strings(GTK_COMBO(combo4), combo_items);
	g_list_free(combo_items);
*/


	gtk_clist_freeze(GTK_CLIST(clist6));
	gtk_clist_clear(GTK_CLIST(clist6));
	for (ln=current_router->interfaces.get_head(); ln!=NULL; ln=ln->next)
	{
		in = (Interface*) ln->item;
		data[0] = strdup("");
		data[1] = strdup(in->descr);
		gtk_clist_append(GTK_CLIST(clist6), data);
		delete data[0]; delete data[1];
		gtk_clist_set_row_data(GTK_CLIST(clist6), row, in);
		if (in->status=='1')
			clist_pixmap(window, GTK_CLIST(clist6), row, 0, (gchar **)intup_xpm);
		else
			clist_pixmap(window, GTK_CLIST(clist6), row, 0, (gchar **)intdown_xpm);
		row++;
	}
	gtk_clist_thaw(GTK_CLIST(clist6));
}

/*
void Stats::interface_list()
{
	Interface *in;
	Link *ln;

	char *string = gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(combo4)->entry));

	for (ln=current_router->interfaces.get_head(); ln!=NULL; ln=ln->next)
	{
		in = (Interface*) ln->item;
		if (strcmp(in->descr, string)==0)
		{
			current_interface = in;
		}
	}
	load();
}
*/

void Stats::start_left()
{
	if (gstart.ts > oldest_time.ts)
	{
		gstart.shift_days(-1);
		day_range++;
	}
	draw();
}

void Stats::start_right()
{
	if ((gstart.ts < gend.ts) && (day_range>1))
	{
		gstart.shift_days(1);
		day_range--;
	}
	draw();
}

void Stats::middle_left()
{
	if (gstart.ts > oldest_time.ts)
	{
		gstart.shift_days(-1);
		gend.shift_days(-1);
	}
	draw();
}

void Stats::middle_right()
{
	if (gend.ts <= newest_time.ts)
	{
		gstart.shift_days(1);
		gend.shift_days(1);
	}
	draw();
}

void Stats::end_left()
{
	if ((gstart.ts < gend.ts) && (day_range>1))
	{
		gend.shift_days(-1);
		day_range--;
	}
	draw();
}

void Stats::end_right()
{
	if (gend.ts <= newest_time.ts)
	{
		gend.shift_days(1);
		day_range++;
	}
	draw();
}

void Stats::load()
{
	char fname[256];

	if ((current_router!=NULL) && (current_interface!=NULL))
	{
		sprintf(fname, "%s/graph/%s/%d", main->session->datadir,
			current_router->pingname.getstring(), current_interface->index);
		if (!gio.get(fname, data))
		{
//			printf("FAILED to load %s\n", fname);
			error = 1;
		}
	}

	draw();
}

void Stats::draw()
{
	GtkStyle *style;
	gint ow, oh;
	int i, j, type, num_speeds, num_times, txtw, day_len, first_day;
	char buf[256], buf2[256], buf3[256];
	GdkFont *fnt;
	gint8 dash_list[2];
	TimeStamp gtmp;

//	struct timeval st, et;
//	gettimeofday(&st, NULL);

	oh = height;
	ow = width;

	if (drawingarea1->window == NULL) return;
	if (data->num_entries == 0) return;

	height = drawingarea1->allocation.height;
	width = drawingarea1->allocation.width;

	if ((canvas == NULL) || (oh != height) || (ow != width))
	{
		if (canvas) gdk_pixmap_unref(canvas);
		canvas = gdk_pixmap_new(drawingarea1->window, width, height, -1);
	}

	fnt = gdk_font_load("-*-helvetica-medium-r-*-*-10-*-*-*-*-*-*-*");

	SX = BX = LEFT_MGN;
	SW = BW = width-LEFT_MGN-RIGHT_MGN;
	SH = (height-TOP_MGN-GAP-BOTTOM_MGN)/4;
	if (SH<40) SH=40;
	if (SH>80) SH=80;
	BH = height-TOP_MGN-GAP-BOTTOM_MGN-SH;
	SY = TOP_MGN;
	BY = TOP_MGN+SH+GAP;

	// work out the time details for the sample data
	oldest_time = data->timestamps[0];
	newest_time = data->timestamps[data->num_entries-1];
	oldest_time.midnight();
	newest_time.midnight();
	length = newest_time.diff_days(oldest_time);
	if (day_range == 0)
	{
		gstart = newest_time;
		gend = newest_time;
		gend.shift_days(1);
	}
	day_range = gend.diff_days(gstart);

	gtmp = gend; gtmp.shift_days(-1);
	first_day = gstart.get_day();
	strcpy(buf2, gstart.date_to_str(2));
	strcpy(buf3, gtmp.date_to_str(2));

// set window title with full dates
	if (day_range>1)
		sprintf(buf, "Network Graph - %s -> %s", buf2, buf3);
	else
		sprintf(buf, "Network Graph - %s", buf2);
	gtk_window_set_title (GTK_WINDOW (window), buf);

// clear window
	gdk_gc_set_foreground(gc, &black);
	style = gtk_widget_get_style(window);
	gtk_draw_flat_box(style, canvas, GTK_STATE_NORMAL,
		GTK_SHADOW_NONE, 0, 0, width, height);

	if ((current_router!=NULL) && (current_interface!=NULL))
	{
		// speeds
		gdk_gc_set_foreground(gc, &black);

		if (scale==0)
		{
			num_speeds = (BH/20)-1;		// 20 is room for text + a gap
			// 1,2,4,5,10,20,25
			if (num_speeds==3) num_speeds=2; else
			if ((num_speeds>=6) && (num_speeds<=9)) num_speeds=5; else
			if ((num_speeds>=11) && (num_speeds<=19)) num_speeds=10; else
			if ((num_speeds>=21) && (num_speeds<=24)) num_speeds=20; else
			if (num_speeds>=26) num_speeds=25;
			for (i=0; i<num_speeds+1; i++)
			{
				speed_to_str((current_interface->speed)*i/num_speeds, buf);
				txtw = gdk_string_width(fnt, buf);
				gdk_draw_string(canvas, fnt, gc, BX-2-txtw, BY+BH-(BH*i/num_speeds)+5, buf);
				gdk_draw_line(canvas, gc, BX-2, BY+BH-(BH*i/num_speeds), BX, BY+BH-(BH*i/num_speeds));
			}
		}else
		{
//			printf("LOG10(6)=%d\n", int(log10(0.5)*1000000));
			num_speeds = (BH/20)-1;		// 20 is room for text + a gap
			// log based scale
		}

		// draw times
		num_times = ((BW/day_range)/14)-1;		// 18 is room for text + a gap

		// 1,2,3,4,6,8,12,24
		if (num_times<=0) num_times=1; else
		if (num_times==5) num_times=4; else
		if (num_times==7) num_times=6; else
		if ((num_times>=9) && (num_times<=11)) num_times=8; else
		if ((num_times>=13) && (num_times<=23)) num_times=12; else
		if (num_times>=25) num_times=24;

		for (i=0; i<(num_times*day_range)+1; i++)
		{
			sprintf(buf, "%d", ((24/num_times)*i)%24);
			txtw = gdk_string_width(fnt, buf);
			gdk_draw_string(canvas, fnt, gc,
				BX-(txtw/2)+(BW*i/(num_times*day_range)), height+12-BOTTOM_MGN, buf);
			gdk_draw_line(canvas, gc, BX+(BW*i/(num_times*day_range)),
				height-BOTTOM_MGN, BX+(BW*i/(num_times*day_range)), height-BOTTOM_MGN+2);
		}

		// work out which format to use from the string lengths
		gtmp.setstring("20000524000000");
		type = 6;
		day_len = (BW/day_range);
		for (i=5; i>1; i--)
		{
			strcpy(buf, gtmp.date_to_str(i));
			if (gdk_string_width(fnt, buf)+4<day_len) type=i;
		}

		// print the days
		gdk_gc_set_foreground(gc, &black);
		for (i=0; i<day_range; i++)
		{
			gtmp = gstart;
			gtmp.shift_days(i);
			strcpy(buf, gtmp.date_to_str(type));
			txtw = gdk_string_width(fnt, buf);
			gdk_draw_string(canvas, fnt, gc,
				BX+((day_len-txtw)/2)+(BW*i/day_range), height+24-BOTTOM_MGN, buf);
		}

		gdk_gc_set_foreground(gridgc, &lightgray);
		for (i=0; i<(num_times*day_range); i++)
		{
			if (i%num_times==0)
			{
				j = (first_day+1+(i/num_times));
				while (j<0) j+=777;
				if ((j%7==0) || (j%7==1))
				{
					gdk_draw_rectangle(canvas, gridgc, TRUE,
						BX+(BW*i/(num_times*day_range)), BY, (BW/day_range)+1, BH);
					gdk_draw_rectangle(canvas, gridgc, TRUE,
						SX+(SW*i/(num_times*day_range)), SY, (SW/day_range)+1, SH);
				}
			}
		}

		// draw grid
		gdk_gc_set_foreground(gridgc, &gray);
		gdk_gc_set_line_attributes (gridgc, 1, GDK_LINE_ON_OFF_DASH,
			GDK_CAP_NOT_LAST, GDK_JOIN_MITER);
		dash_list[0] = 2; dash_list[1] = 2;
		gdk_gc_set_dashes(gridgc, 0, dash_list, sizeof(dash_list));
		for (i=1; i<(num_times*day_range); i++)
		{
			gdk_draw_line(canvas, gridgc, BX+(BW*i/(num_times*day_range)), BY+BH,
				BX+(BW*i/(num_times*day_range)), BY);
			gdk_draw_line(canvas, gridgc, SX+(SW*i/(num_times*day_range)), SY+SH,
				SX+(SW*i/(num_times*day_range)), SY);
		}
		for (i=1; i<num_speeds; i++)
		{
			gdk_draw_line(canvas, gridgc, BX, BY+BH-(BH*i/num_speeds), BX+BW,
				BY+BH-(BH*i/num_speeds));
		}

		gdk_gc_set_foreground(gridgc, &darkgray);
		dash_list[0] = 4; dash_list[1] = 4;
		gdk_gc_set_dashes(gridgc, 0, dash_list, sizeof(dash_list));
		for (i=0; i<(num_times*day_range); i++)
		{
			if (i%num_times==0)
			{
				gdk_draw_line(canvas, gridgc, BX+(BW*i/(num_times*day_range)),
					BY+BH, BX+(BW*i/(num_times*day_range)), BY);
				gdk_draw_line(canvas, gridgc, SX+(SW*i/(num_times*day_range)),
					SY+SH, SX+(SW*i/(num_times*day_range)), SY);
			}
		}

		// draw lines
		for (i=0; i<data->num_variables; i++)
		{
			draw_line(i, 20);
		}

		// draw axis
		gdk_gc_set_foreground(gc, &black);
		gdk_draw_rectangle(canvas, gc, FALSE, BX, BY, BW, BH);
		gdk_draw_rectangle(canvas, gc, FALSE, SX, SY, SW, SH);
	}else
	{
		gdk_gc_set_foreground(gc, &black);
		if (error == 1)
			strcpy(buf, "Failed to load data file");
		else
			strcpy(buf, "Error");
		txtw = gdk_string_width(fnt, buf);
		gdk_draw_string(canvas, fnt, gc, (width-txtw)/2, height/2, buf);
	}

	gdk_draw_pixmap(drawingarea1->window,
		drawingarea1->style->fg_gc[GTK_WIDGET_STATE(drawingarea1)],
		canvas, 0, 0, 0, 0, drawingarea1->allocation.width,
		drawingarea1->allocation.height);

//	gettimeofday(&et, NULL);
//	unsigned long diff = (1000000*(et.tv_sec-st.tv_sec))+(et.tv_usec-st.tv_usec);
//	printf("Stats::draw : %lu\n", diff);
}

void Stats::draw_line(int var, int scale)
{
	unsigned long val1, val2;

	TimeStamp tx1, tx2;
	gint i, x1, y1, x2, y2;
	int x, y, w, h;
	GdkRectangle rect;
	GdkColor col;

	if (current_interface->speed==0) return;

	x=BX; y=BY; w=BW; h=BH;

	// set line colour
	alloc_color(col, gio.global_types[var].r,
		gio.global_types[var].g, gio.global_types[var].b);
	gdk_gc_set_foreground(gc, &col);

	for (i=1; i<data->num_entries; i++)
	{
		// work out coordinates in the graph
		val1 = data->variables[var].values[i-1];
		val2 = data->variables[var].values[i];
		tx1 = data->timestamps[i-1];
		tx2 = data->timestamps[i];
		if ((tx1.ts>gstart.ts) && (tx1.ts<gend.ts))
		{
			x1 = ((tx1.ts-gstart.ts)*w/(gend.ts-gstart.ts))+x;
			x2 = ((tx2.ts-gstart.ts)*w/(gend.ts-gstart.ts))+x;
			if (gio.global_types[var].rate == 1)
			{
				y1 = y+h-((val1*h*8)/current_interface->speed);
				y2 = y+h-((val2*h*8)/current_interface->speed);
				rect.x = x; rect.y = y;
				rect.width = w; rect.height = h;
				gdk_gc_set_clip_rectangle(gc, &rect);
				gdk_draw_line(canvas, gc, x1, y1, x2, y2);
				gdk_gc_set_clip_rectangle(gc, NULL);
			}else
			if (gio.global_types[var].rate == 2)
			{
				// draw in top box
				y2 = ((val2*SH)/100);
//				rect.x = SX; rect.y = SY;
//				rect.width = SW; rect.height = SH;
//				gdk_gc_set_clip_rectangle(gc, &rect);
				gdk_draw_rectangle(canvas, gc, TRUE, x1, SY+SH-y2, x2-x1, y2);
//				gdk_draw_line(canvas, gc, x1, SY+SH, x2, SY+SH-y2);
//printf ("Y2 %lu %d\n", val2, y2);
//				gdk_gc_set_clip_rectangle(gc, NULL);
			}
		}
	}
}

