/* graphio.h
 * Copyright (C) 2001 The University of Newcastle upon Tyne
 *
 * Written by Paul Coates (Paul.Coates@ncl.ac.uk)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 */

#ifndef GRAPHIO_H
#define GRAPHIO_H 1

#include "dataset.h"
#include "header.h"

class GraphBlock
{
	public:
		GraphBlock();

		unsigned long gap;				// 5 minute intervals
		unsigned long howlong;		// upto 2 weeks
		unsigned long length;			// the block is x bytes long
		unsigned long start;			// start of block is x bytes into the file
		unsigned long offset;			// the next record should be written here
		char wrapped;
};

// for real snmp oid values you would need 512 bytes but since we are
// restricting the graphs to monitor interface counters this limit has
// been reduced

class GraphType
{
	public:
		GraphType();

		char oid[65];
		char label[65];
		int rate;
		int r, g, b;
};

class GraphIO
{
	public:
		GraphIO();
		~GraphIO();

		void init_global();
		void init_local();
		int write_global();
		int read_global();
		int valid();
		int update(char *fname);
		void load_header(FILE *f);
		int create(char *fname);
		int add(char *fname);
		int get(char *fname, DataSet *d);

		char datadir[256];

		// these are global values
		GraphBlock global_blocks[MAX_BLOCKS];
		int num_global_blocks;
		int num_global_values;
		GraphType global_types[MAX_VALUES];

		// these are local values
		GraphBlock blocks[MAX_BLOCKS];
		int num_blocks;
		int num_values;
		GraphType types[MAX_VALUES];

		// fill these in before calling add
		TimeStamp current;
		char status;
		unsigned long value[MAX_VALUES];

	private:
		void add_recursive(FILE *f, int lvl);
		void get_record(DataSet *d, FILE *f, int count);
};

#endif

