#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include "ipaddress.h"

IPAddress::IPAddress()
{
	empty();
}

IPAddress::~IPAddress()
{
}

IPAddress::IPAddress(IPAddress &orig)
{
	ip = orig.ip;
}

void IPAddress::operator = (const IPAddress &orig)
{
	ip = orig.ip;
}

void IPAddress::empty()
{
	ip = 0;
}

void IPAddress::print()
{
	printf("%s", getstring());
}

unsigned long IPAddress::getul()
{
	return ip;
}

void IPAddress::setul(unsigned long l)
{
	ip = l;
}

char *IPAddress::getstring()
{
	static char buf[16];
	int a, b, c, d;

	a = (ip & 0xff000000) >> 24;
	b = (ip & 0xff0000) >> 16;
	c = (ip & 0xff00) >> 8;
	d = (ip & 0xff);

	sprintf(buf, "%d.%d.%d.%d", a, b, c, d);
	return buf;
}

void IPAddress::setstring(char *str)
{
	int a, b, c, d;
	if (sscanf(str, "%d.%d.%d.%d", &a, &b, &c, &d)!=4) return;
	ip = (((((a*256)+b)*256)+c)*256)+d;
}

void IPAddress::setpartialstring(char *str)
{
	int a, b, c, d;

	if (sscanf(str, "%d.%d.%d.%d", &a, &b, &c, &d)==4)
		ip = (((((a*256)+b)*256)+c)*256)+d;
	else
	if ((sscanf(str, "%d.%d.%d", &a, &b, &c)==3) ||
			(sscanf(str, "%d.%d.%d.", &a, &b, &c)==3))
		ip = ((((a*256)+b)*256)+c)*256;
	else
	if ((sscanf(str, "%d.%d", &a, &b)==2) ||
			(sscanf(str, "%d.%d.", &a, &b)==2))
		ip = ((a*256)+b)*256*256;
	else
	if ((sscanf(str, "%d", &a)==1) ||
			(sscanf(str, "%d.", &a)==1))
		ip = a*256*256*256;
}

// convert 0100007F into 127.0.0.1
void IPAddress::setafinet(char *str)
{
	char buf[4];
	int i, v[4];

	if (strlen(str)!=8) return;
	empty();
	for (i=0; i<4; i++)
	{
		buf[0] = str[(i*2)]; buf[1] = str[(i*2)+1]; buf[2] = '\0';
		if (sscanf(buf, "%x", &v[i])!=1) return;
	}
	ip = (((((v[3]*256)+v[2])*256)+v[1])*256)+v[0];
}

char *IPAddress::getuls()
{
	static char buf[16];
	sprintf(buf, "%lu", ip);
	return buf;
}

void IPAddress::setuls(char *buf)
{
	ip = strtoul(buf, NULL, 10);
}

void IPAddress::set(IPAddress pi)
{
	ip = pi.ip;
}

int IPAddress::zero()
{
	return (ip==0);
}

int IPAddress::compare(IPAddress pi)
{
	return (ip-pi.ip);
}

int IPAddress::compare(char *str)
{
	IPAddress pi;
	pi.setstring(str);
	return (ip-pi.ip);
}

int IPAddress::in_range(IPAddress s1, IPAddress s2)
{
	return ((s1.ip<=ip) && (s2.ip>=ip));
}

int IPAddress::in_range(unsigned long s1, unsigned long s2)
{
	return ((s1<=ip) && (s2>=ip));
}

int IPAddress::in_range(char *str1, char *str2)
{
	IPAddress s1, s2;
	s1.setstring(str1);
	s2.setstring(str2);
	return ((s1.ip<=ip) && (s2.ip>=ip));
}

void IPAddress::make_ip(IPAddress subnet, int mask, int offset)
{
	if (mask==0) { ip=0; return; }
	unsigned long val = mask_power(mask);
	ip = (subnet.ip/val)*val;
	ip += offset;
}

void IPAddress::make_ip(char *subnet, int mask, int offset)
{
	if (mask==0) { ip=0; return; }
	unsigned long val = mask_power(mask);
	setstring(subnet);
	ip = (ip/val)*val;
	ip += offset;
}

void IPAddress::make_subnet(IPAddress s1, int mask)
{
	make_ip(s1, mask, 0);
}

void IPAddress::make_subnet(char *str, int mask)
{
	make_ip(str, mask, 0);
}

void IPAddress::make_broadcast(IPAddress s1, int mask)
{
	if (mask==0) { ip=0; return; }
	unsigned long val = mask_power(mask);
	ip = s1.ip + val - 1;
}

void IPAddress::make_mask(int mask)	// convert 24 into 255.255.255.0
{
	ip = 0xffffffff - mask_power(mask) + 1;
}

int IPAddress::mask_value()	// convert 255.255.255.0 into 24
{
	int count = 0;
	IPAddress total;

	total.ip = ip;
	if (total.ip == 0) return 32;
	while (total.ip%2 == 0) { total.ip /= 2; count++; }
	return 32-count;
}

int mask_power(int pwr)	// convert 24 into 256, 30 into 4, etc.
{
	int i, current = 1;

	pwr = 32-pwr;
	if (pwr<0) return -1;
	if (pwr==0) return 1;
	for (i=0; i<pwr; i++) current *= 2;
	return current;
}

int overlap(IPAddress ip1, int m1, IPAddress ip2, int m2)
{
	int mask1, mask2;

	mask1 = mask_power(m1);
	mask2 = mask_power(m2);

	if ((ip1.ip+mask1-1 < ip2.ip) || (ip2.ip+mask2-1 < ip1.ip))
		return 0;
	return 1;
}

int IPAddress::valid_mask()
{
	unsigned long val = 0xffffffff;
	int i, dg, found = 0;

	dg = 1;
	for (i=0; i<32; i++)
	{
		if (val==ip) found=1;
		val -= dg;
		dg *= 2;
	}
	return found;
}

